%% Austria %%
%==========================================================================
% housekeeping
%==========================================================================
clear
close all;
userpath('clear');


rng('default'); % reinitialize the random number generator to its startup configuration
rng(0);         % set seed

currdir=pwd;
cd ..
get_help_dir_currdir=pwd;
addpath([get_help_dir_currdir,'/helpfunctions']); % set path to helper functions
cd(currdir)

%==========================================================================
% load the data and priors
%==========================================================================
data  = readtable([get_help_dir_currdir,'/data/csvfiles/dataset.csv']);
data   = data(find(strcmp(data.dates,'1970-01-01')==1):find(strcmp(data.dates,'2007-06-01')==1),:);
[Austria,~,~] = xlsread('Austria.xls','Sheet1','B122:H571');
exc_au = Austria(:,1);     
ip_au  = Austria(:,2);    
cpi_au = Austria(:,3);     
ir_au  = Austria(:,4);     
tb_au = Austria(:,7);     
[us,~,~] = xlsread('us.xls','dataset','B62:I511');
ip_us = 100*log(us(:,7));
cpi_us= 100*log(us(:,8));
cpr_us= 100*log(us(:,3));
tr_us= 100*log(us(:,4));
bog_us= 100*log(us(:,5));
ffr= us(:,6);

num  = [ip_us cpi_us cpr_us tr_us bog_us ffr exc_au ip_au cpi_au ir_au tb_au];

%==========================================================================
% model setup
%==========================================================================
nlag      = 2;                  % number of lags, HQ
nvar      = 11;                   % number of endogenous variables
nex       = 0;                   % set equal to 1 if a constant is included; 0 otherwise
m         = nvar*nlag + nex;     % number of exogenous variables
nd        = 500000;                % number of orthogonal-reduced-form (B,Sigma,Q) draws
iter_show = 5e2;                 % display iteration every iter_show draws
horizon   = 60;                  % maximum horizon for IRFs
index     = 40;                  % define  horizons for the FEVD
horizons  = 0:5;                 % horizons to restrict
NS        = 1 + numel(horizons); % number of objects in F(A_{0},A_{+}) to which we impose sign and zero restrictios: F(THETA)=[A_{0};L_{0},...,L_{horizons}]
e         = eye(nvar);           % create identity matrix
maxdraws  = 1e4;                 % max number of importance sampling draws
conjugate = 'structural';        % structural or irfs or empty
%==========================================================================
% identification: declare Ss and Zs matrices
%==========================================================================
% restrictions on A0  and/or IRFs
% sign restrictions
S = cell(nvar,1);
for ii=1:nvar
    S{ii}=zeros(0,nvar*NS);
end


S{1}(1,nvar+2) = -1;S{1}(5,2*nvar+2)  = -1;S{1}(9,3*nvar+2)  = -1;S{1}(13,4*nvar+2) = -1;S{1}(17,5*nvar+2) = -1;S{1}(21,6*nvar+2) = -1;
S{1}(2,nvar+3) = -1;S{1}(6,2*nvar+3)  = -1;S{1}(10,3*nvar+3) = -1;S{1}(14,4*nvar+3) = -1;S{1}(18,5*nvar+3) = -1;S{1}(22,6*nvar+3) = -1;
S{1}(3,nvar+5) = -1;S{1}(7,2*nvar+5)  = -1;S{1}(11,3*nvar+5) = -1;S{1}(15,4*nvar+5) = -1;S{1}(19,5*nvar+5) = -1;S{1}(23,6*nvar+5) = -1;
S{1}(4,nvar+6) =  1;S{1}(8,2*nvar+6)  =  1;S{1}(12,3*nvar+6) =  1;S{1}(16,4*nvar+6) =  1;S{1}(20,5*nvar+6) =  1;S{1}(24,6*nvar+6) =  1;
S{1}(25,1)        = -1;
S{1}(26,2)        = -1;
S{1}(27,3)        = -1;
S{1}(28,6)        =  1;

% zero restrictions
Z=cell(nvar,1);
for i=1:nvar
    Z{i}=zeros(0,nvar*NS);
end
Z{1}(1,4)=1;
Z{1}(2,5)=1;
Z{1}(3,7)=1; 
Z{1}(4,8)=1; 
Z{1}(5,9)=1; 
Z{1}(6,10)=1;    Z{1}(7,11)=1; 

%==========================================================================
% Setup info
%==========================================================================
info=SetupInfo(nvar,m,Z,@(x)chol(x));

% ZF(A_{0},A_{+})
info.nlag     = nlag;
info.horizons = horizons;
info.ZF       = @(x,y)ZF(x,y);

% functions useful to compute the importance sampler weights
iw_info = info;
fs      = @(x)ff_h(x,iw_info);
r       = @(x)ZeroRestrictions(x,iw_info);

if strcmp(conjugate,'irfs')==1
    fo              = @(x)f_h(x,iw_info);
    fo_str2irfs     = @(x)StructuralToIRF(x,iw_info);
    fo_str2irfs_inv = @(x)IRFToStructural(x,iw_info);
    r_irfs          = @(x)IRFRestrictions_more_general(x,iw_info); 
end

% function useful to check the sign restrictions
fh_S_restrictions  = @(x)SF(x,iw_info,S);

%==========================================================================
% write data in Rubio, Waggoner, and Zha (RES 2010)'s notation
%==========================================================================
% yt(t) A0 = xt(t) Aplus + constant + et(t) for t=1...,T;
% yt(t)    = xt(t) B     + ut(t)            for t=1...,T;
% x(t)     = [yt(t-1), ... , yt(t-nlag), constant];
% matrix notation yt = xt*B + ut;
% xt=[yt_{-1} ones(T,1)];
yt = num(nlag+1:end,:);
T  = size(yt,1);
xt = zeros(T,nvar*nlag+nex);
for i=1:nlag
    xt(:,nvar*(i-1)+1:nvar*i) = num((nlag-(i-1)):end-i,:) ;
end
if nex>=1
    xt(:,nvar*nlag+nex)=ones(T,1);
end
% write data in Zellner (1971, pp 224-227) notation
Y = yt; % T by nvar matrix of observations
X = xt; % T by (nvar*nlag+1) matrix of regressors

% prior for reduced-form parameters
nnuBar              = 0;
OomegaBarInverse    = zeros(m);
PpsiBar             = zeros(m,nvar);
PphiBar             = zeros(nvar);

% posterior for reduced-form parameters
nnuTilde            = T +nnuBar;
OomegaTilde         = (X'*X  + OomegaBarInverse)\eye(m);
OomegaTildeInverse  =  X'*X  + OomegaBarInverse;
PpsiTilde           = OomegaTilde*(X'*Y + OomegaBarInverse*PpsiBar);
PphiTilde           = Y'*Y + PphiBar + PpsiBar'*OomegaBarInverse*PpsiBar - PpsiTilde'*OomegaTildeInverse*PpsiTilde;
PphiTilde           = (PphiTilde'+PphiTilde)*0.5;

% useful definitions
% definitios used to store orthogonal-reduced-form draws, volume elements, and unnormalized weights
Bdraws         = cell([nd,1]); % reduced-form lag parameters
Sigmadraws     = cell([nd,1]); % reduced-form covariance matrices
Qdraws         = cell([nd,1]); % orthogonal matrices
storevefh      = zeros(nd,1);  % volume element f_{h}
storevegfhZ    = zeros(nd,1);  % volume element g o f_{h}|Z
uw             = zeros(nd,1);  % unnormalized importance sampler weights

if strcmp(conjugate,'irfs')==1
    storevephi      = zeros(nd,1);  % volume element f_{h}
    storevegphiZ    = zeros(nd,1);  % volume element g o f_{h}|Z
end

% definitions related to IRFs; based on page 12 of Rubio, Waggoner, and Zha (RES 2010)
J      = [e;repmat(zeros(nvar),nlag-1,1)];
A      = cell(nlag,1);
extraF = repmat(zeros(nvar),1,nlag-1);
F      = zeros(nlag*nvar,nlag*nvar);
for l=1:nlag-1
    F((l-1)*nvar+1:l*nvar,nvar+1:nlag*nvar)=[repmat(zeros(nvar),1,l-1) e repmat(zeros(nvar),1,nlag-(l+1))];
end

% definition to facilitate the draws from B|Sigma
hh              = info.h;
cholOomegaTilde = hh(OomegaTilde)'; % this matrix is used to draw B|Sigma below

% initialize counters to track the state of the computations
counter = 1;
record  = 1;
count   = 0;
tStart = tic;
while record<=nd
    
    % step 1 in Algorithm 2
    Sigmadraw     = iwishrnd(PphiTilde,nnuTilde);
    cholSigmadraw = hh(Sigmadraw)';
    Bdraw         = kron(cholSigmadraw,cholOomegaTilde)*randn(m*nvar,1) + reshape(PpsiTilde,nvar*m,1);
    Bdraw         = reshape(Bdraw,nvar*nlag+nex,nvar);
    % store reduced-form draws
    Bdraws{record,1}     = Bdraw;
    Sigmadraws{record,1} = Sigmadraw;
    
    % steps 2:4 of Algorithm 2
    w           = DrawW(iw_info);   
    x           = [vec(Bdraw); vec(Sigmadraw); w];
    structpara  = ff_h_inv(x,iw_info);
    
    % store the matrix Q associated with step 3
    Qdraw            = SpheresToQ(w,iw_info,Bdraw,Sigmadraw);
    Qdraws{record,1} = reshape(Qdraw,nvar,nvar);
    
    % check if sign restrictions hold
    signs      = fh_S_restrictions(structpara);
    
    if (sum(signs{1}*e(:,1)>0))==size(signs{1}*e(:,1),1)
        
        count=count+1;
        
        % compute importance sampling weights
       
        switch conjugate
            
            case 'structural'
                
                storevefh(record,1)   = (nvar*(nvar+1)/2)*log(2)-(2*nvar+m+1)*LogAbsDet(reshape(structpara(1:nvar*nvar),nvar,nvar));
                storevegfhZ(record,1) = LogVolumeElement(fs,structpara,r); 
                uw(record,1)          = exp(storevefh(record,1) - storevegfhZ(record,1));
                
            case 'irfs'
                
                irfpara                = fo_str2irfs(structpara);
                storevephi(record,1)   = LogVolumeElement(fo,structpara)   + LogVolumeElement(fo_str2irfs_inv,irfpara);
                storevegphiZ(record,1) = LogVolumeElement(fs,structpara,r) + LogVolumeElement(fo_str2irfs_inv,irfpara,r_irfs); 
                uw(record,1)           = exp(storevephi(record,1) - storevegphiZ(record,1));
                
            otherwise
                
                uw(record,1) = 1;
                
        end
        
    else
        
        uw(record,1) = 0;
        
    end
    
    if counter==iter_show
        
        display(['Number of draws = ',num2str(record)])
        display(['Remaining draws = ',num2str(nd-(record))])
        display(['1. Austria'])
        counter =0;
        
    end
    counter = counter + 1;
    record=record+1;
end

tElapsed = toc(tStart);
imp_w  = uw/sum(uw);
ne = floor(1/sum(imp_w.^2));

% store draws
Ltilde        = zeros(horizon+1,nvar,nvar,ne); % define array to store IRF
A0tilde       = zeros(nvar,nvar,ne); % define array to store A0
Aplustilde    = zeros(m,nvar,ne); % define array to store Aplus
FEVD          = zeros(nvar,size(index,2),ne);                 % define array to store FEVD
hist_is_draws = zeros(ne,1);   % define array to store draws from importance sampler

for s=1:min(ne,maxdraws)
    
    % draw: B,Sigma,Q
    is_draw = randsample(1:size(imp_w,1),1,true,imp_w);
    hist_is_draws(s,1)=is_draw;

    Bdraw       = Bdraws{is_draw,1};
    Sigmadraw   = Sigmadraws{is_draw,1};
    Qdraw       = Qdraws{is_draw,1};
    
    % compute matrix F: useful for FEVD
    hSigmadraw = hh(Sigmadraw);
    A0         = hSigmadraw\e;
    Aplus      = Bdraw*A0;
    % Page 8 ARRW
    for l=1:nlag-1
        A{l} = Aplus((l-1)*nvar+1:l*nvar,1:end);
        F((l-1)*nvar+1:l*nvar,1:nvar)=A{l}/A0;
    end
    A{nlag} = Aplus((nlag-1)*nvar+1:nlag*nvar,1:end);
    F((nlag-1)*nvar+1:nlag*nvar,:)=[A{nlag}/A0 extraF];
    
    x=[reshape(Bdraw,m*nvar,1); reshape(Sigmadraw,nvar*nvar,1); Qdraw(:)];
    structpara = f_h_inv(x,info);
    
    LIRF =IRF_horizons(structpara, nvar, nlag, m, 0:horizon);
    
    for h=0:horizon
        Ltilde(h+1,:,:,s) =  LIRF(1+h*nvar:(h+1)*nvar,:);
    end
    
    for j=1:size(index,2)
    FEVD(:,j,s)         = variancedecomposition(F',J,Sigmadraw, hh(Sigmadraw)'*Qdraw(:,1),nvar,index(1,j));
    end
    
    A0tilde(:,:,s)    = reshape(structpara(1:nvar*nvar),nvar,nvar);
    Aplustilde(:,:,s) = reshape(structpara(nvar*nvar+1:end),m,nvar);
end

A0tilde    = A0tilde(:,:,1:s);
Aplustilde = Aplustilde(:,:,1:s);
Ltilde     = Ltilde(:,:,:,1:s);
FEVD       = FEVD(:,:,1:s);

cd results
savefile='results_au.mat';
save(savefile,'Ltilde','A0tilde','Aplustilde','imp_w','ne','FEVD');
cd ..



%% CANADA %%
%==========================================================================
% housekeeping
%==========================================================================
clearvars -except usmpshock* 
close all;
userpath('clear');


rng('default'); % reinitialize the random number generator to its startup configuration
rng(0);         % set seed

currdir=pwd;
cd ..
get_help_dir_currdir=pwd;
addpath([get_help_dir_currdir,'/helpfunctions']); % set path to helper functions
cd(currdir)

%==========================================================================
% load the data and priors
%==========================================================================
data  = readtable([get_help_dir_currdir,'/data/csvfiles/dataset.csv']);
data   = data(find(strcmp(data.dates,'1970-01-01')==1):find(strcmp(data.dates,'2007-06-01')==1),:);
[Canada,~,~] = xlsread('Canada.xls','Sheet1','B122:H571');
exc_ca = Canada(:,1);     % exchange rate
ip_ca  = Canada(:,2);     % Un
cpi_ca = Canada(:,3);     % CPI
ir_ca  = Canada(:,4);     % short term interest rate
tb_ca = Canada(:,7);     % exports
[us,~,~] = xlsread('us.xls','dataset','B62:I511');
ip_us = 100*log(us(:,7));
cpi_us= 100*log(us(:,8));
cpr_us= 100*log(us(:,3));
tr_us= 100*log(us(:,4));
bog_us= 100*log(us(:,5));
ffr= us(:,6);

num  = [ip_us cpi_us cpr_us tr_us bog_us ffr exc_ca ip_ca cpi_ca ir_ca tb_ca];

%==========================================================================
% model setup
%==========================================================================
nlag      = 2;                  % number of lags
nvar      = 11;                   % number of endogenous variables
nex       = 0;                   % set equal to 1 if a constant is included; 0 otherwise
m         = nvar*nlag + nex;     % number of exogenous variables
nd        = 500000;                % number of orthogonal-reduced-form (B,Sigma,Q) draws
iter_show = 5e2;                 % display iteration every iter_show draws
horizon   = 60;                  % maximum horizon for IRFs
index     = 40;                  % define  horizons for the FEVD
horizons  = 0:5;                 % horizons to restrict
NS        = 1 + numel(horizons); % number of objects in F(A_{0},A_{+}) to which we impose sign and zero restrictios: F(THETA)=[A_{0};L_{0},...,L_{horizons}]
e         = eye(nvar);           % create identity matrix
maxdraws  = 1e4;                 % max number of importance sampling draws
conjugate = 'structural';        % structural or irfs or empty
%==========================================================================
% identification: declare Ss and Zs matrices
%==========================================================================
% restrictions on A0  and/or IRFs
% sign restrictions
S = cell(nvar,1);
for ii=1:nvar
    S{ii}=zeros(0,nvar*NS);
end

S{1}(1,nvar+2) = -1;S{1}(5,2*nvar+2)  = -1;S{1}(9,3*nvar+2)  = -1;S{1}(13,4*nvar+2) = -1;S{1}(17,5*nvar+2) = -1;S{1}(21,6*nvar+2) = -1;
S{1}(2,nvar+3) = -1;S{1}(6,2*nvar+3)  = -1;S{1}(10,3*nvar+3) = -1;S{1}(14,4*nvar+3) = -1;S{1}(18,5*nvar+3) = -1;S{1}(22,6*nvar+3) = -1;
S{1}(3,nvar+5) = -1;S{1}(7,2*nvar+5)  = -1;S{1}(11,3*nvar+5) = -1;S{1}(15,4*nvar+5) = -1;S{1}(19,5*nvar+5) = -1;S{1}(23,6*nvar+5) = -1;
S{1}(4,nvar+6) =  1;S{1}(8,2*nvar+6)  =  1;S{1}(12,3*nvar+6) =  1;S{1}(16,4*nvar+6) =  1;S{1}(20,5*nvar+6) =  1;S{1}(24,6*nvar+6) =  1;
S{1}(25,1)        = -1;
S{1}(26,2)        = -1;
S{1}(27,3)        = -1;
S{1}(28,6)        =  1;

% zero restrictions
Z=cell(nvar,1);
for i=1:nvar
    Z{i}=zeros(0,nvar*NS);
end
Z{1}(1,4)=1;
Z{1}(2,5)=1;
Z{1}(3,7)=1; 
Z{1}(4,8)=1; 
Z{1}(5,9)=1; 
Z{1}(6,10)=1;    Z{1}(7,11)=1; 

%==========================================================================
% Setup info
%==========================================================================
info=SetupInfo(nvar,m,Z,@(x)chol(x));

% ZF(A_{0},A_{+})
info.nlag     = nlag;
info.horizons = horizons;
info.ZF       = @(x,y)ZF(x,y);

% functions useful to compute the importance sampler weights
iw_info = info;
fs      = @(x)ff_h(x,iw_info);
r       = @(x)ZeroRestrictions(x,iw_info);

if strcmp(conjugate,'irfs')==1
    fo              = @(x)f_h(x,iw_info);
    fo_str2irfs     = @(x)StructuralToIRF(x,iw_info);
    fo_str2irfs_inv = @(x)IRFToStructural(x,iw_info);
    r_irfs          = @(x)IRFRestrictions_more_general(x,iw_info); 
end

% function useful to check the sign restrictions
fh_S_restrictions  = @(x)SF(x,iw_info,S);

%==========================================================================
% write data in Rubio, Waggoner, and Zha (RES 2010)'s notation
%==========================================================================
% yt(t) A0 = xt(t) Aplus + constant + et(t) for t=1...,T;
% yt(t)    = xt(t) B     + ut(t)            for t=1...,T;
% x(t)     = [yt(t-1), ... , yt(t-nlag), constant];
% matrix notation yt = xt*B + ut;
% xt=[yt_{-1} ones(T,1)];
yt = num(nlag+1:end,:);
T  = size(yt,1);
xt = zeros(T,nvar*nlag+nex);
for i=1:nlag
    xt(:,nvar*(i-1)+1:nvar*i) = num((nlag-(i-1)):end-i,:) ;
end
if nex>=1
    xt(:,nvar*nlag+nex)=ones(T,1);
end
% write data in Zellner (1971, pp 224-227) notation
Y = yt; % T by nvar matrix of observations
X = xt; % T by (nvar*nlag+1) matrix of regressors

% prior for reduced-form parameters
nnuBar              = 0;
OomegaBarInverse    = zeros(m);
PpsiBar             = zeros(m,nvar);
PphiBar             = zeros(nvar);

% posterior for reduced-form parameters
nnuTilde            = T +nnuBar;
OomegaTilde         = (X'*X  + OomegaBarInverse)\eye(m);
OomegaTildeInverse  =  X'*X  + OomegaBarInverse;
PpsiTilde           = OomegaTilde*(X'*Y + OomegaBarInverse*PpsiBar);
PphiTilde           = Y'*Y + PphiBar + PpsiBar'*OomegaBarInverse*PpsiBar - PpsiTilde'*OomegaTildeInverse*PpsiTilde;
PphiTilde           = (PphiTilde'+PphiTilde)*0.5;

% useful definitions
% definitios used to store orthogonal-reduced-form draws, volume elements, and unnormalized weights
Bdraws         = cell([nd,1]); % reduced-form lag parameters
Sigmadraws     = cell([nd,1]); % reduced-form covariance matrices
Qdraws         = cell([nd,1]); % orthogonal matrices
storevefh      = zeros(nd,1);  % volume element f_{h}
storevegfhZ    = zeros(nd,1);  % volume element g o f_{h}|Z
uw             = zeros(nd,1);  % unnormalized importance sampler weights

if strcmp(conjugate,'irfs')==1
    storevephi      = zeros(nd,1);  % volume element f_{h}
    storevegphiZ    = zeros(nd,1);  % volume element g o f_{h}|Z
end

% definitions related to IRFs; based on page 12 of Rubio, Waggoner, and Zha (RES 2010)
J      = [e;repmat(zeros(nvar),nlag-1,1)];
A      = cell(nlag,1);
extraF = repmat(zeros(nvar),1,nlag-1);
F      = zeros(nlag*nvar,nlag*nvar);
for l=1:nlag-1
    F((l-1)*nvar+1:l*nvar,nvar+1:nlag*nvar)=[repmat(zeros(nvar),1,l-1) e repmat(zeros(nvar),1,nlag-(l+1))];
end

% definition to facilitate the draws from B|Sigma
hh              = info.h;
cholOomegaTilde = hh(OomegaTilde)'; % this matrix is used to draw B|Sigma below

% initialize counters to track the state of the computations
counter = 1;
record  = 1;
count   = 0;
tStart = tic;
while record<=nd
    
    % step 1 in Algorithm 2
    Sigmadraw     = iwishrnd(PphiTilde,nnuTilde);
    cholSigmadraw = hh(Sigmadraw)';
    Bdraw         = kron(cholSigmadraw,cholOomegaTilde)*randn(m*nvar,1) + reshape(PpsiTilde,nvar*m,1);
    Bdraw         = reshape(Bdraw,nvar*nlag+nex,nvar);
    % store reduced-form draws
    Bdraws{record,1}     = Bdraw;
    Sigmadraws{record,1} = Sigmadraw;
    
    % steps 2:4 of Algorithm 2
    w           = DrawW(iw_info);   
    x           = [vec(Bdraw); vec(Sigmadraw); w];
    structpara  = ff_h_inv(x,iw_info);
    
    % store the matrix Q associated with step 3
    Qdraw            = SpheresToQ(w,iw_info,Bdraw,Sigmadraw);
    Qdraws{record,1} = reshape(Qdraw,nvar,nvar);
    
    % check if sign restrictions hold
    signs      = fh_S_restrictions(structpara);
    
    if (sum(signs{1}*e(:,1)>0))==size(signs{1}*e(:,1),1)
        
        count=count+1;
        
        % compute importance sampling weights
       
        switch conjugate
            
            case 'structural'
                
                storevefh(record,1)   = (nvar*(nvar+1)/2)*log(2)-(2*nvar+m+1)*LogAbsDet(reshape(structpara(1:nvar*nvar),nvar,nvar));
                storevegfhZ(record,1) = LogVolumeElement(fs,structpara,r); 
                uw(record,1)          = exp(storevefh(record,1) - storevegfhZ(record,1));
                
            case 'irfs'
                
                irfpara                = fo_str2irfs(structpara);
                storevephi(record,1)   = LogVolumeElement(fo,structpara)   + LogVolumeElement(fo_str2irfs_inv,irfpara);
                storevegphiZ(record,1) = LogVolumeElement(fs,structpara,r) + LogVolumeElement(fo_str2irfs_inv,irfpara,r_irfs); 
                uw(record,1)           = exp(storevephi(record,1) - storevegphiZ(record,1));
                
            otherwise
                
                uw(record,1) = 1;
                
        end
        
    else
        
        uw(record,1) = 0;
        
    end
    
    if counter==iter_show
        
        display(['Number of draws = ',num2str(record)])
        display(['Remaining draws = ',num2str(nd-(record))])
        display(['2. CANADA'])
        counter =0;
        
    end
    counter = counter + 1;
    record=record+1;
end

tElapsed = toc(tStart);
imp_w  = uw/sum(uw);
ne = floor(1/sum(imp_w.^2));

% store draws
Ltilde        = zeros(horizon+1,nvar,nvar,ne); % define array to store IRF
A0tilde       = zeros(nvar,nvar,ne); % define array to store A0
Aplustilde    = zeros(m,nvar,ne); % define array to store Aplus
FEVD          = zeros(nvar,size(index,2),ne);                 % define array to store FEVD
hist_is_draws = zeros(ne,1);   % define array to store draws from importance sampler

for s=1:min(ne,maxdraws)
    
    % draw: B,Sigma,Q
    is_draw = randsample(1:size(imp_w,1),1,true,imp_w);
    hist_is_draws(s,1)=is_draw;

    Bdraw       = Bdraws{is_draw,1};
    Sigmadraw   = Sigmadraws{is_draw,1};
    Qdraw       = Qdraws{is_draw,1};
    
    % compute matrix F: useful for FEVD
    hSigmadraw = hh(Sigmadraw);
    A0         = hSigmadraw\e;
    Aplus      = Bdraw*A0;
    % Page 8 ARRW
    for l=1:nlag-1
        A{l} = Aplus((l-1)*nvar+1:l*nvar,1:end);
        F((l-1)*nvar+1:l*nvar,1:nvar)=A{l}/A0;
    end
    A{nlag} = Aplus((nlag-1)*nvar+1:nlag*nvar,1:end);
    F((nlag-1)*nvar+1:nlag*nvar,:)=[A{nlag}/A0 extraF];
    
    x=[reshape(Bdraw,m*nvar,1); reshape(Sigmadraw,nvar*nvar,1); Qdraw(:)];
    structpara = f_h_inv(x,info);
    
    LIRF =IRF_horizons(structpara, nvar, nlag, m, 0:horizon);
    
    for h=0:horizon
        Ltilde(h+1,:,:,s) =  LIRF(1+h*nvar:(h+1)*nvar,:);
    end
    
    for j=1:size(index,2)
    FEVD(:,j,s)         = variancedecomposition(F',J,Sigmadraw, hh(Sigmadraw)'*Qdraw(:,1),nvar,index(1,j));
    end
    
    A0tilde(:,:,s)    = reshape(structpara(1:nvar*nvar),nvar,nvar);
    Aplustilde(:,:,s) = reshape(structpara(nvar*nvar+1:end),m,nvar);
end

A0tilde    = A0tilde(:,:,1:s);
Aplustilde = Aplustilde(:,:,1:s);
Ltilde     = Ltilde(:,:,:,1:s);
FEVD       = FEVD(:,:,1:s);

cd results
savefile='results_ca.mat';
save(savefile,'Ltilde','A0tilde','Aplustilde','imp_w','ne','FEVD');
cd ..



%% Denmark %%%
%==========================================================================
% housekeeping
%==========================================================================
clearvars -except usmpshock* 
close all;
userpath('clear');


rng('default'); % reinitialize the random number generator to its startup configuration
rng(0);         % set seed

currdir=pwd;
cd ..
get_help_dir_currdir=pwd;
addpath([get_help_dir_currdir,'/helpfunctions']); % set path to helper functions
cd(currdir)

%==========================================================================
% load the data and priors
%==========================================================================
data  = readtable([get_help_dir_currdir,'/data/csvfiles/dataset.csv']);
data   = data(find(strcmp(data.dates,'1974-01-01')==1):find(strcmp(data.dates,'2007-06-01')==1),:);
[Denmark,~,~] = xlsread('Denmark.xls','Sheet1','B170:H571');
exc_de = Denmark(:,1);     % exchange rate
ip_de  = Denmark(:,2);     % IP
cpi_de = Denmark(:,3);     % CPI
ir_de  = Denmark(:,4);     % short term interest rate
tb_de  = Denmark(:,7);     %
[us,~,~] = xlsread('us.xls','dataset','B110:I511');
ip_us = 100*log(us(:,7));
cpi_us= 100*log(us(:,8));
cpr_us= 100*log(us(:,3));
tr_us= 100*log(us(:,4));
bog_us= 100*log(us(:,5));
ffr= us(:,6);
num  = [ip_us cpi_us cpr_us tr_us bog_us ffr exc_de ip_de cpi_de ir_de tb_de];

%==========================================================================
% model setup
%==========================================================================
nlag      = 1;                  % number of lags
nvar      = 11;                   % number of endogenous variables
nex       = 0;                   % set equal to 1 if a constant is included; 0 otherwise
m         = nvar*nlag + nex;     % number of exogenous variables
nd        = 500000;                % number of orthogonal-reduced-form (B,Sigma,Q) draws
iter_show = 5e2;                 % display iteration every iter_show draws
horizon   = 60;                  % maximum horizon for IRFs
index     = 40;                  % define  horizons for the FEVD
horizons  = 0:5;                 % horizons to restrict
NS        = 1 + numel(horizons); % number of objects in F(A_{0},A_{+}) to which we impose sign and zero restrictios: F(THETA)=[A_{0};L_{0},...,L_{horizons}]
e         = eye(nvar);           % create identity matrix
maxdraws  = 1e4;                 % max number of importance sampling draws
conjugate = 'structural';        % structural or irfs or empty
%==========================================================================
% identification: declare Ss and Zs matrices
%==========================================================================
% restrictions on A0  and/or IRFs
% sign restrictions
S = cell(nvar,1);
for ii=1:nvar
    S{ii}=zeros(0,nvar*NS);
end

S{1}(1,nvar+2) = -1;S{1}(5,2*nvar+2)  = -1;S{1}(9,3*nvar+2)  = -1;S{1}(13,4*nvar+2) = -1;S{1}(17,5*nvar+2) = -1;S{1}(21,6*nvar+2) = -1;
S{1}(2,nvar+3) = -1;S{1}(6,2*nvar+3)  = -1;S{1}(10,3*nvar+3) = -1;S{1}(14,4*nvar+3) = -1;S{1}(18,5*nvar+3) = -1;S{1}(22,6*nvar+3) = -1;
S{1}(3,nvar+5) = -1;S{1}(7,2*nvar+5)  = -1;S{1}(11,3*nvar+5) = -1;S{1}(15,4*nvar+5) = -1;S{1}(19,5*nvar+5) = -1;S{1}(23,6*nvar+5) = -1;
S{1}(4,nvar+6) =  1;S{1}(8,2*nvar+6)  =  1;S{1}(12,3*nvar+6) =  1;S{1}(16,4*nvar+6) =  1;S{1}(20,5*nvar+6) =  1;S{1}(24,6*nvar+6) =  1;
S{1}(25,1)        = -1;
S{1}(26,2)        = -1;
S{1}(27,3)        = -1;
S{1}(28,6)        =  1;

% zero restrictions
Z=cell(nvar,1);
for i=1:nvar
    Z{i}=zeros(0,nvar*NS);
end
Z{1}(1,4)=1;
Z{1}(2,5)=1;
Z{1}(3,7)=1; 
Z{1}(4,8)=1; 
Z{1}(5,9)=1; 
Z{1}(6,10)=1;    Z{1}(7,11)=1; 

%==========================================================================
% Setup info
%==========================================================================
info=SetupInfo(nvar,m,Z,@(x)chol(x));

% ZF(A_{0},A_{+})
info.nlag     = nlag;
info.horizons = horizons;
info.ZF       = @(x,y)ZF(x,y);

% functions useful to compute the importance sampler weights
iw_info = info;
fs      = @(x)ff_h(x,iw_info);
r       = @(x)ZeroRestrictions(x,iw_info);

if strcmp(conjugate,'irfs')==1
    fo              = @(x)f_h(x,iw_info);
    fo_str2irfs     = @(x)StructuralToIRF(x,iw_info);
    fo_str2irfs_inv = @(x)IRFToStructural(x,iw_info);
    r_irfs          = @(x)IRFRestrictions_more_general(x,iw_info); 
end

% function useful to check the sign restrictions
fh_S_restrictions  = @(x)SF(x,iw_info,S);

%==========================================================================
% write data in Rubio, Waggoner, and Zha (RES 2010)'s notation
%==========================================================================
% yt(t) A0 = xt(t) Aplus + constant + et(t) for t=1...,T;
% yt(t)    = xt(t) B     + ut(t)            for t=1...,T;
% x(t)     = [yt(t-1), ... , yt(t-nlag), constant];
% matrix notation yt = xt*B + ut;
% xt=[yt_{-1} ones(T,1)];
yt = num(nlag+1:end,:);
T  = size(yt,1);
xt = zeros(T,nvar*nlag+nex);
for i=1:nlag
    xt(:,nvar*(i-1)+1:nvar*i) = num((nlag-(i-1)):end-i,:) ;
end
if nex>=1
    xt(:,nvar*nlag+nex)=ones(T,1);
end
% write data in Zellner (1971, pp 224-227) notation
Y = yt; % T by nvar matrix of observations
X = xt; % T by (nvar*nlag+1) matrix of regressors

% prior for reduced-form parameters
nnuBar              = 0;
OomegaBarInverse    = zeros(m);
PpsiBar             = zeros(m,nvar);
PphiBar             = zeros(nvar);

% posterior for reduced-form parameters
nnuTilde            = T +nnuBar;
OomegaTilde         = (X'*X  + OomegaBarInverse)\eye(m);
OomegaTildeInverse  =  X'*X  + OomegaBarInverse;
PpsiTilde           = OomegaTilde*(X'*Y + OomegaBarInverse*PpsiBar);
PphiTilde           = Y'*Y + PphiBar + PpsiBar'*OomegaBarInverse*PpsiBar - PpsiTilde'*OomegaTildeInverse*PpsiTilde;
PphiTilde           = (PphiTilde'+PphiTilde)*0.5;

% useful definitions
% definitios used to store orthogonal-reduced-form draws, volume elements, and unnormalized weights
Bdraws         = cell([nd,1]); % reduced-form lag parameters
Sigmadraws     = cell([nd,1]); % reduced-form covariance matrices
Qdraws         = cell([nd,1]); % orthogonal matrices
storevefh      = zeros(nd,1);  % volume element f_{h}
storevegfhZ    = zeros(nd,1);  % volume element g o f_{h}|Z
uw             = zeros(nd,1);  % unnormalized importance sampler weights

if strcmp(conjugate,'irfs')==1
    storevephi      = zeros(nd,1);  % volume element f_{h}
    storevegphiZ    = zeros(nd,1);  % volume element g o f_{h}|Z
end

% definitions related to IRFs; based on page 12 of Rubio, Waggoner, and Zha (RES 2010)
J      = [e;repmat(zeros(nvar),nlag-1,1)];
A      = cell(nlag,1);
extraF = repmat(zeros(nvar),1,nlag-1);
F      = zeros(nlag*nvar,nlag*nvar);
for l=1:nlag-1
    F((l-1)*nvar+1:l*nvar,nvar+1:nlag*nvar)=[repmat(zeros(nvar),1,l-1) e repmat(zeros(nvar),1,nlag-(l+1))];
end

% definition to facilitate the draws from B|Sigma
hh              = info.h;
cholOomegaTilde = hh(OomegaTilde)'; % this matrix is used to draw B|Sigma below

% initialize counters to track the state of the computations
counter = 1;
record  = 1;
count   = 0;
tStart = tic;
while record<=nd
    
    % step 1 in Algorithm 2
    Sigmadraw     = iwishrnd(PphiTilde,nnuTilde);
    cholSigmadraw = hh(Sigmadraw)';
    Bdraw         = kron(cholSigmadraw,cholOomegaTilde)*randn(m*nvar,1) + reshape(PpsiTilde,nvar*m,1);
    Bdraw         = reshape(Bdraw,nvar*nlag+nex,nvar);
    % store reduced-form draws
    Bdraws{record,1}     = Bdraw;
    Sigmadraws{record,1} = Sigmadraw;
    
    % steps 2:4 of Algorithm 2
    w           = DrawW(iw_info);   
    x           = [vec(Bdraw); vec(Sigmadraw); w];
    structpara  = ff_h_inv(x,iw_info);
    
    % store the matrix Q associated with step 3
    Qdraw            = SpheresToQ(w,iw_info,Bdraw,Sigmadraw);
    Qdraws{record,1} = reshape(Qdraw,nvar,nvar);
    
    % check if sign restrictions hold
    signs      = fh_S_restrictions(structpara);
    
    if (sum(signs{1}*e(:,1)>0))==size(signs{1}*e(:,1),1)
        
        count=count+1;
        
        % compute importance sampling weights
       
        switch conjugate
            
            case 'structural'
                
                storevefh(record,1)   = (nvar*(nvar+1)/2)*log(2)-(2*nvar+m+1)*LogAbsDet(reshape(structpara(1:nvar*nvar),nvar,nvar));
                storevegfhZ(record,1) = LogVolumeElement(fs,structpara,r); 
                uw(record,1)          = exp(storevefh(record,1) - storevegfhZ(record,1));
                
            case 'irfs'
                
                irfpara                = fo_str2irfs(structpara);
                storevephi(record,1)   = LogVolumeElement(fo,structpara)   + LogVolumeElement(fo_str2irfs_inv,irfpara);
                storevegphiZ(record,1) = LogVolumeElement(fs,structpara,r) + LogVolumeElement(fo_str2irfs_inv,irfpara,r_irfs); 
                uw(record,1)           = exp(storevephi(record,1) - storevegphiZ(record,1));
                
            otherwise
                
                uw(record,1) = 1;
                
        end
        
    else
        
        uw(record,1) = 0;
        
    end
    
    if counter==iter_show
        
        display(['Number of draws = ',num2str(record)])
        display(['Remaining draws = ',num2str(nd-(record))])
        display(['3. Denmark'])
        counter =0;
        
    end
    counter = counter + 1;
    record=record+1;
end

tElapsed = toc(tStart);
imp_w  = uw/sum(uw);
ne = floor(1/sum(imp_w.^2));

% store draws
Ltilde        = zeros(horizon+1,nvar,nvar,ne); % define array to store IRF
A0tilde       = zeros(nvar,nvar,ne); % define array to store A0
Aplustilde    = zeros(m,nvar,ne); % define array to store Aplus
FEVD          = zeros(nvar,size(index,2),ne);                 % define array to store FEVD
hist_is_draws = zeros(ne,1);   % define array to store draws from importance sampler

for s=1:min(ne,maxdraws)
    
    % draw: B,Sigma,Q
    is_draw = randsample(1:size(imp_w,1),1,true,imp_w);
    hist_is_draws(s,1)=is_draw;

    Bdraw       = Bdraws{is_draw,1};
    Sigmadraw   = Sigmadraws{is_draw,1};
    Qdraw       = Qdraws{is_draw,1};
    
    % compute matrix F: useful for FEVD
    hSigmadraw = hh(Sigmadraw);
    A0         = hSigmadraw\e;
    Aplus      = Bdraw*A0;
    % Page 8 ARRW
    for l=1:nlag-1
        A{l} = Aplus((l-1)*nvar+1:l*nvar,1:end);
        F((l-1)*nvar+1:l*nvar,1:nvar)=A{l}/A0;
    end
    A{nlag} = Aplus((nlag-1)*nvar+1:nlag*nvar,1:end);
    F((nlag-1)*nvar+1:nlag*nvar,:)=[A{nlag}/A0 extraF];
    
    x=[reshape(Bdraw,m*nvar,1); reshape(Sigmadraw,nvar*nvar,1); Qdraw(:)];
    structpara = f_h_inv(x,info);
    
    LIRF =IRF_horizons(structpara, nvar, nlag, m, 0:horizon);
    
    for h=0:horizon
        Ltilde(h+1,:,:,s) =  LIRF(1+h*nvar:(h+1)*nvar,:);
    end
    
    for j=1:size(index,2)
    FEVD(:,j,s)         = variancedecomposition(F',J,Sigmadraw, hh(Sigmadraw)'*Qdraw(:,1),nvar,index(1,j));
    end
    
    A0tilde(:,:,s)    = reshape(structpara(1:nvar*nvar),nvar,nvar);
    Aplustilde(:,:,s) = reshape(structpara(nvar*nvar+1:end),m,nvar);
end

A0tilde    = A0tilde(:,:,1:s);
Aplustilde = Aplustilde(:,:,1:s);
Ltilde     = Ltilde(:,:,:,1:s);
FEVD       = FEVD(:,:,1:s);

cd results
savefile='results_de.mat';
save(savefile,'Ltilde','A0tilde','Aplustilde','imp_w','ne','FEVD');
cd ..



%% Finland %%%
%==========================================================================
% housekeeping
%==========================================================================
clearvars -except usmpshock* 
close all;
userpath('clear');


rng('default'); % reinitialize the random number generator to its startup configuration
rng(0);         % set seed

currdir=pwd;
cd ..
get_help_dir_currdir=pwd;
addpath([get_help_dir_currdir,'/helpfunctions']); % set path to helper functions
cd(currdir)

%==========================================================================
% load the data and priors
%==========================================================================
data  = readtable([get_help_dir_currdir,'/data/csvfiles/dataset.csv']);
data   = data(find(strcmp(data.dates,'1970-01-01')==1):find(strcmp(data.dates,'2007-06-01')==1),:);
[Finland,~,~] = xlsread('Finland.xls','Sheet1','B122:H571');
exc_fi = Finland(:,1);     % exchange rate
ip_fi  = Finland(:,2);     % IP
cpi_fi = Finland(:,3);     % CPI
ir_fi  = Finland(:,4);     % short term interest rate
tb_fi  = Finland(:,7);     % tb
[us,~,~] = xlsread('us.xls','dataset','B62:I511');
ip_us = 100*log(us(:,7));
cpi_us= 100*log(us(:,8));
cpr_us= 100*log(us(:,3));
tr_us= 100*log(us(:,4));
bog_us= 100*log(us(:,5));
ffr= us(:,6);
num  = [ip_us cpi_us cpr_us tr_us bog_us ffr exc_fi ip_fi cpi_fi ir_fi tb_fi];

%==========================================================================
% model setup
%==========================================================================
nlag      = 3;                  % number of lags
nvar      = 11;                   % number of endogenous variables
nex       = 0;                   % set equal to 1 if a constant is included; 0 otherwise
m         = nvar*nlag + nex;     % number of exogenous variables
nd        = 500000;                % number of orthogonal-reduced-form (B,Sigma,Q) draws
iter_show = 5e2;                 % display iteration every iter_show draws
horizon   = 60;                  % maximum horizon for IRFs
index     = 40;                  % define  horizons for the FEVD
horizons  = 0:5;                 % horizons to restrict
NS        = 1 + numel(horizons); % number of objects in F(A_{0},A_{+}) to which we impose sign and zero restrictios: F(THETA)=[A_{0};L_{0},...,L_{horizons}]
e         = eye(nvar);           % create identity matrix
maxdraws  = 1e4;                 % max number of importance sampling draws
conjugate = 'structural';        % structural or irfs or empty
%==========================================================================
% identification: declare Ss and Zs matrices
%==========================================================================
% restrictions on A0  and/or IRFs
% sign restrictions
S = cell(nvar,1);
for ii=1:nvar
    S{ii}=zeros(0,nvar*NS);
end

S{1}(1,nvar+2) = -1;S{1}(5,2*nvar+2)  = -1;S{1}(9,3*nvar+2)  = -1;S{1}(13,4*nvar+2) = -1;S{1}(17,5*nvar+2) = -1;S{1}(21,6*nvar+2) = -1;
S{1}(2,nvar+3) = -1;S{1}(6,2*nvar+3)  = -1;S{1}(10,3*nvar+3) = -1;S{1}(14,4*nvar+3) = -1;S{1}(18,5*nvar+3) = -1;S{1}(22,6*nvar+3) = -1;
S{1}(3,nvar+5) = -1;S{1}(7,2*nvar+5)  = -1;S{1}(11,3*nvar+5) = -1;S{1}(15,4*nvar+5) = -1;S{1}(19,5*nvar+5) = -1;S{1}(23,6*nvar+5) = -1;
S{1}(4,nvar+6) =  1;S{1}(8,2*nvar+6)  =  1;S{1}(12,3*nvar+6) =  1;S{1}(16,4*nvar+6) =  1;S{1}(20,5*nvar+6) =  1;S{1}(24,6*nvar+6) =  1;
S{1}(25,1)        = -1;
S{1}(26,2)        = -1;
S{1}(27,3)        = -1;
S{1}(28,6)        =  1;

% zero restrictions
Z=cell(nvar,1);
for i=1:nvar
    Z{i}=zeros(0,nvar*NS);
end
Z{1}(1,4)=1;
Z{1}(2,5)=1;
Z{1}(3,7)=1; 
Z{1}(4,8)=1; 
Z{1}(5,9)=1; 
Z{1}(6,10)=1;    Z{1}(7,11)=1; 

%==========================================================================
% Setup info
%==========================================================================
info=SetupInfo(nvar,m,Z,@(x)chol(x));

% ZF(A_{0},A_{+})
info.nlag     = nlag;
info.horizons = horizons;
info.ZF       = @(x,y)ZF(x,y);

% functions useful to compute the importance sampler weights
iw_info = info;
fs      = @(x)ff_h(x,iw_info);
r       = @(x)ZeroRestrictions(x,iw_info);

if strcmp(conjugate,'irfs')==1
    fo              = @(x)f_h(x,iw_info);
    fo_str2irfs     = @(x)StructuralToIRF(x,iw_info);
    fo_str2irfs_inv = @(x)IRFToStructural(x,iw_info);
    r_irfs          = @(x)IRFRestrictions_more_general(x,iw_info); 
end

% function useful to check the sign restrictions
fh_S_restrictions  = @(x)SF(x,iw_info,S);

%==========================================================================
% write data in Rubio, Waggoner, and Zha (RES 2010)'s notation
%==========================================================================
% yt(t) A0 = xt(t) Aplus + constant + et(t) for t=1...,T;
% yt(t)    = xt(t) B     + ut(t)            for t=1...,T;
% x(t)     = [yt(t-1), ... , yt(t-nlag), constant];
% matrix notation yt = xt*B + ut;
% xt=[yt_{-1} ones(T,1)];
yt = num(nlag+1:end,:);
T  = size(yt,1);
xt = zeros(T,nvar*nlag+nex);
for i=1:nlag
    xt(:,nvar*(i-1)+1:nvar*i) = num((nlag-(i-1)):end-i,:) ;
end
if nex>=1
    xt(:,nvar*nlag+nex)=ones(T,1);
end
% write data in Zellner (1971, pp 224-227) notation
Y = yt; % T by nvar matrix of observations
X = xt; % T by (nvar*nlag+1) matrix of regressors

% prior for reduced-form parameters
nnuBar              = 0;
OomegaBarInverse    = zeros(m);
PpsiBar             = zeros(m,nvar);
PphiBar             = zeros(nvar);

% posterior for reduced-form parameters
nnuTilde            = T +nnuBar;
OomegaTilde         = (X'*X  + OomegaBarInverse)\eye(m);
OomegaTildeInverse  =  X'*X  + OomegaBarInverse;
PpsiTilde           = OomegaTilde*(X'*Y + OomegaBarInverse*PpsiBar);
PphiTilde           = Y'*Y + PphiBar + PpsiBar'*OomegaBarInverse*PpsiBar - PpsiTilde'*OomegaTildeInverse*PpsiTilde;
PphiTilde           = (PphiTilde'+PphiTilde)*0.5;

% useful definitions
% definitios used to store orthogonal-reduced-form draws, volume elements, and unnormalized weights
Bdraws         = cell([nd,1]); % reduced-form lag parameters
Sigmadraws     = cell([nd,1]); % reduced-form covariance matrices
Qdraws         = cell([nd,1]); % orthogonal matrices
storevefh      = zeros(nd,1);  % volume element f_{h}
storevegfhZ    = zeros(nd,1);  % volume element g o f_{h}|Z
uw             = zeros(nd,1);  % unnormalized importance sampler weights

if strcmp(conjugate,'irfs')==1
    storevephi      = zeros(nd,1);  % volume element f_{h}
    storevegphiZ    = zeros(nd,1);  % volume element g o f_{h}|Z
end

% definitions related to IRFs; based on page 12 of Rubio, Waggoner, and Zha (RES 2010)
J      = [e;repmat(zeros(nvar),nlag-1,1)];
A      = cell(nlag,1);
extraF = repmat(zeros(nvar),1,nlag-1);
F      = zeros(nlag*nvar,nlag*nvar);
for l=1:nlag-1
    F((l-1)*nvar+1:l*nvar,nvar+1:nlag*nvar)=[repmat(zeros(nvar),1,l-1) e repmat(zeros(nvar),1,nlag-(l+1))];
end

% definition to facilitate the draws from B|Sigma
hh              = info.h;
cholOomegaTilde = hh(OomegaTilde)'; % this matrix is used to draw B|Sigma below

% initialize counters to track the state of the computations
counter = 1;
record  = 1;
count   = 0;
tStart = tic;
while record<=nd
    
    % step 1 in Algorithm 2
    Sigmadraw     = iwishrnd(PphiTilde,nnuTilde);
    cholSigmadraw = hh(Sigmadraw)';
    Bdraw         = kron(cholSigmadraw,cholOomegaTilde)*randn(m*nvar,1) + reshape(PpsiTilde,nvar*m,1);
    Bdraw         = reshape(Bdraw,nvar*nlag+nex,nvar);
    % store reduced-form draws
    Bdraws{record,1}     = Bdraw;
    Sigmadraws{record,1} = Sigmadraw;
    
    % steps 2:4 of Algorithm 2
    w           = DrawW(iw_info);   
    x           = [vec(Bdraw); vec(Sigmadraw); w];
    structpara  = ff_h_inv(x,iw_info);
    
    % store the matrix Q associated with step 3
    Qdraw            = SpheresToQ(w,iw_info,Bdraw,Sigmadraw);
    Qdraws{record,1} = reshape(Qdraw,nvar,nvar);
    
    % check if sign restrictions hold
    signs      = fh_S_restrictions(structpara);
    
    if (sum(signs{1}*e(:,1)>0))==size(signs{1}*e(:,1),1)
        
        count=count+1;
        
        % compute importance sampling weights
       
        switch conjugate
            
            case 'structural'
                
                storevefh(record,1)   = (nvar*(nvar+1)/2)*log(2)-(2*nvar+m+1)*LogAbsDet(reshape(structpara(1:nvar*nvar),nvar,nvar));
                storevegfhZ(record,1) = LogVolumeElement(fs,structpara,r); 
                uw(record,1)          = exp(storevefh(record,1) - storevegfhZ(record,1));
                
            case 'irfs'
                
                irfpara                = fo_str2irfs(structpara);
                storevephi(record,1)   = LogVolumeElement(fo,structpara)   + LogVolumeElement(fo_str2irfs_inv,irfpara);
                storevegphiZ(record,1) = LogVolumeElement(fs,structpara,r) + LogVolumeElement(fo_str2irfs_inv,irfpara,r_irfs); 
                uw(record,1)           = exp(storevephi(record,1) - storevegphiZ(record,1));
                
            otherwise
                
                uw(record,1) = 1;
                
        end
        
    else
        
        uw(record,1) = 0;
        
    end
    
    if counter==iter_show
        
        display(['Number of draws = ',num2str(record)])
        display(['Remaining draws = ',num2str(nd-(record))])
        display(['4. Finland'])
        counter =0;
        
    end
    counter = counter + 1;
    record=record+1;
end

tElapsed = toc(tStart);
imp_w  = uw/sum(uw);
ne = floor(1/sum(imp_w.^2));

% store draws
Ltilde        = zeros(horizon+1,nvar,nvar,ne); % define array to store IRF
A0tilde       = zeros(nvar,nvar,ne); % define array to store A0
Aplustilde    = zeros(m,nvar,ne); % define array to store Aplus
FEVD          = zeros(nvar,size(index,2),ne);                 % define array to store FEVD
hist_is_draws = zeros(ne,1);   % define array to store draws from importance sampler

for s=1:min(ne,maxdraws)
    
    % draw: B,Sigma,Q
    is_draw = randsample(1:size(imp_w,1),1,true,imp_w);
    hist_is_draws(s,1)=is_draw;

    Bdraw       = Bdraws{is_draw,1};
    Sigmadraw   = Sigmadraws{is_draw,1};
    Qdraw       = Qdraws{is_draw,1};
    
    % compute matrix F: useful for FEVD
    hSigmadraw = hh(Sigmadraw);
    A0         = hSigmadraw\e;
    Aplus      = Bdraw*A0;
    % Page 8 ARRW
    for l=1:nlag-1
        A{l} = Aplus((l-1)*nvar+1:l*nvar,1:end);
        F((l-1)*nvar+1:l*nvar,1:nvar)=A{l}/A0;
    end
    A{nlag} = Aplus((nlag-1)*nvar+1:nlag*nvar,1:end);
    F((nlag-1)*nvar+1:nlag*nvar,:)=[A{nlag}/A0 extraF];
    
    x=[reshape(Bdraw,m*nvar,1); reshape(Sigmadraw,nvar*nvar,1); Qdraw(:)];
    structpara = f_h_inv(x,info);
    
    LIRF =IRF_horizons(structpara, nvar, nlag, m, 0:horizon);
    
    for h=0:horizon
        Ltilde(h+1,:,:,s) =  LIRF(1+h*nvar:(h+1)*nvar,:);
    end
    
    for j=1:size(index,2)
    FEVD(:,j,s)         = variancedecomposition(F',J,Sigmadraw, hh(Sigmadraw)'*Qdraw(:,1),nvar,index(1,j));
    end
    
    A0tilde(:,:,s)    = reshape(structpara(1:nvar*nvar),nvar,nvar);
    Aplustilde(:,:,s) = reshape(structpara(nvar*nvar+1:end),m,nvar);
end

A0tilde    = A0tilde(:,:,1:s);
Aplustilde = Aplustilde(:,:,1:s);
Ltilde     = Ltilde(:,:,:,1:s);
FEVD       = FEVD(:,:,1:s);

cd results
savefile='results_fi.mat';
save(savefile,'Ltilde','A0tilde','Aplustilde','imp_w','ne','FEVD');
cd ..



%% France %%
%==========================================================================
% housekeeping
%==========================================================================
clearvars -except usmpshock* 
close all;
userpath('clear');


rng('default'); % reinitialize the random number generator to its startup configuration
rng(0);         % set seed

currdir=pwd;
cd ..
get_help_dir_currdir=pwd;
addpath([get_help_dir_currdir,'/helpfunctions']); % set path to helper functions
cd(currdir)

%==========================================================================
% load the data and priors
%==========================================================================
data  = readtable([get_help_dir_currdir,'/data/csvfiles/dataset.csv']);
data   = data(find(strcmp(data.dates,'1970-01-01')==1):find(strcmp(data.dates,'2007-06-01')==1),:);
[France,~,~] = xlsread('France.xls','Sheet1','B122:H571');
exc_fr = France(:,1);     % exchange rate
ip_fr  = France(:,2);     % Un
cpi_fr = France(:,3);     % CPI
ir_fr  = France(:,4);     % short term interest rate
tb_fr = France(:,7);     % tb
[us,~,~] = xlsread('us.xls','dataset','B62:I511');
ip_us = 100*log(us(:,7));
cpi_us= 100*log(us(:,8));
cpr_us= 100*log(us(:,3));
tr_us= 100*log(us(:,4));
bog_us= 100*log(us(:,5));
ffr= us(:,6);
num  = [ip_us cpi_us cpr_us tr_us bog_us ffr exc_fr ip_fr cpi_fr ir_fr tb_fr];

%==========================================================================
% model setup
%==========================================================================
nlag      = 2;                  % number of lags
nvar      = 11;                   % number of endogenous variables
nex       = 0;                   % set equal to 1 if a constant is included; 0 otherwise
m         = nvar*nlag + nex;     % number of exogenous variables
nd        = 500000;                % number of orthogonal-reduced-form (B,Sigma,Q) draws
iter_show = 5e2;                 % display iteration every iter_show draws
horizon   = 60;                  % maximum horizon for IRFs
index     = 40;                  % define  horizons for the FEVD
horizons  = 0:5;                 % horizons to restrict
NS        = 1 + numel(horizons); % number of objects in F(A_{0},A_{+}) to which we impose sign and zero restrictios: F(THETA)=[A_{0};L_{0},...,L_{horizons}]
e         = eye(nvar);           % create identity matrix
maxdraws  = 1e4;                 % max number of importance sampling draws
conjugate = 'structural';        % structural or irfs or empty
%==========================================================================
% identification: declare Ss and Zs matrices
%==========================================================================
% restrictions on A0  and/or IRFs
% sign restrictions
S = cell(nvar,1);
for ii=1:nvar
    S{ii}=zeros(0,nvar*NS);
end

S{1}(1,nvar+2) = -1;S{1}(5,2*nvar+2)  = -1;S{1}(9,3*nvar+2)  = -1;S{1}(13,4*nvar+2) = -1;S{1}(17,5*nvar+2) = -1;S{1}(21,6*nvar+2) = -1;
S{1}(2,nvar+3) = -1;S{1}(6,2*nvar+3)  = -1;S{1}(10,3*nvar+3) = -1;S{1}(14,4*nvar+3) = -1;S{1}(18,5*nvar+3) = -1;S{1}(22,6*nvar+3) = -1;
S{1}(3,nvar+5) = -1;S{1}(7,2*nvar+5)  = -1;S{1}(11,3*nvar+5) = -1;S{1}(15,4*nvar+5) = -1;S{1}(19,5*nvar+5) = -1;S{1}(23,6*nvar+5) = -1;
S{1}(4,nvar+6) =  1;S{1}(8,2*nvar+6)  =  1;S{1}(12,3*nvar+6) =  1;S{1}(16,4*nvar+6) =  1;S{1}(20,5*nvar+6) =  1;S{1}(24,6*nvar+6) =  1;
S{1}(25,1)        = -1;
S{1}(26,2)        = -1;
S{1}(27,3)        = -1;
S{1}(28,6)        =  1;

% zero restrictions
Z=cell(nvar,1);
for i=1:nvar
    Z{i}=zeros(0,nvar*NS);
end
Z{1}(1,4)=1;
Z{1}(2,5)=1;
Z{1}(3,7)=1; 
Z{1}(4,8)=1; 
Z{1}(5,9)=1; 
Z{1}(6,10)=1;    Z{1}(7,11)=1; 

%==========================================================================
% Setup info
%==========================================================================
info=SetupInfo(nvar,m,Z,@(x)chol(x));

% ZF(A_{0},A_{+})
info.nlag     = nlag;
info.horizons = horizons;
info.ZF       = @(x,y)ZF(x,y);

% functions useful to compute the importance sampler weights
iw_info = info;
fs      = @(x)ff_h(x,iw_info);
r       = @(x)ZeroRestrictions(x,iw_info);

if strcmp(conjugate,'irfs')==1
    fo              = @(x)f_h(x,iw_info);
    fo_str2irfs     = @(x)StructuralToIRF(x,iw_info);
    fo_str2irfs_inv = @(x)IRFToStructural(x,iw_info);
    r_irfs          = @(x)IRFRestrictions_more_general(x,iw_info); 
end

% function useful to check the sign restrictions
fh_S_restrictions  = @(x)SF(x,iw_info,S);

%==========================================================================
% write data in Rubio, Waggoner, and Zha (RES 2010)'s notation
%==========================================================================
% yt(t) A0 = xt(t) Aplus + constant + et(t) for t=1...,T;
% yt(t)    = xt(t) B     + ut(t)            for t=1...,T;
% x(t)     = [yt(t-1), ... , yt(t-nlag), constant];
% matrix notation yt = xt*B + ut;
% xt=[yt_{-1} ones(T,1)];
yt = num(nlag+1:end,:);
T  = size(yt,1);
xt = zeros(T,nvar*nlag+nex);
for i=1:nlag
    xt(:,nvar*(i-1)+1:nvar*i) = num((nlag-(i-1)):end-i,:) ;
end
if nex>=1
    xt(:,nvar*nlag+nex)=ones(T,1);
end
% write data in Zellner (1971, pp 224-227) notation
Y = yt; % T by nvar matrix of observations
X = xt; % T by (nvar*nlag+1) matrix of regressors

% prior for reduced-form parameters
nnuBar              = 0;
OomegaBarInverse    = zeros(m);
PpsiBar             = zeros(m,nvar);
PphiBar             = zeros(nvar);

% posterior for reduced-form parameters
nnuTilde            = T +nnuBar;
OomegaTilde         = (X'*X  + OomegaBarInverse)\eye(m);
OomegaTildeInverse  =  X'*X  + OomegaBarInverse;
PpsiTilde           = OomegaTilde*(X'*Y + OomegaBarInverse*PpsiBar);
PphiTilde           = Y'*Y + PphiBar + PpsiBar'*OomegaBarInverse*PpsiBar - PpsiTilde'*OomegaTildeInverse*PpsiTilde;
PphiTilde           = (PphiTilde'+PphiTilde)*0.5;

% useful definitions
% definitios used to store orthogonal-reduced-form draws, volume elements, and unnormalized weights
Bdraws         = cell([nd,1]); % reduced-form lag parameters
Sigmadraws     = cell([nd,1]); % reduced-form covariance matrices
Qdraws         = cell([nd,1]); % orthogonal matrices
storevefh      = zeros(nd,1);  % volume element f_{h}
storevegfhZ    = zeros(nd,1);  % volume element g o f_{h}|Z
uw             = zeros(nd,1);  % unnormalized importance sampler weights

if strcmp(conjugate,'irfs')==1
    storevephi      = zeros(nd,1);  % volume element f_{h}
    storevegphiZ    = zeros(nd,1);  % volume element g o f_{h}|Z
end

% definitions related to IRFs; based on page 12 of Rubio, Waggoner, and Zha (RES 2010)
J      = [e;repmat(zeros(nvar),nlag-1,1)];
A      = cell(nlag,1);
extraF = repmat(zeros(nvar),1,nlag-1);
F      = zeros(nlag*nvar,nlag*nvar);
for l=1:nlag-1
    F((l-1)*nvar+1:l*nvar,nvar+1:nlag*nvar)=[repmat(zeros(nvar),1,l-1) e repmat(zeros(nvar),1,nlag-(l+1))];
end

% definition to facilitate the draws from B|Sigma
hh              = info.h;
cholOomegaTilde = hh(OomegaTilde)'; % this matrix is used to draw B|Sigma below

% initialize counters to track the state of the computations
counter = 1;
record  = 1;
count   = 0;
tStart = tic;
while record<=nd
    
    % step 1 in Algorithm 2
    Sigmadraw     = iwishrnd(PphiTilde,nnuTilde);
    cholSigmadraw = hh(Sigmadraw)';
    Bdraw         = kron(cholSigmadraw,cholOomegaTilde)*randn(m*nvar,1) + reshape(PpsiTilde,nvar*m,1);
    Bdraw         = reshape(Bdraw,nvar*nlag+nex,nvar);
    % store reduced-form draws
    Bdraws{record,1}     = Bdraw;
    Sigmadraws{record,1} = Sigmadraw;
    
    % steps 2:4 of Algorithm 2
    w           = DrawW(iw_info);   
    x           = [vec(Bdraw); vec(Sigmadraw); w];
    structpara  = ff_h_inv(x,iw_info);
    
    % store the matrix Q associated with step 3
    Qdraw            = SpheresToQ(w,iw_info,Bdraw,Sigmadraw);
    Qdraws{record,1} = reshape(Qdraw,nvar,nvar);
    
    % check if sign restrictions hold
    signs      = fh_S_restrictions(structpara);
    
    if (sum(signs{1}*e(:,1)>0))==size(signs{1}*e(:,1),1)
        
        count=count+1;
        
        % compute importance sampling weights
       
        switch conjugate
            
            case 'structural'
                
                storevefh(record,1)   = (nvar*(nvar+1)/2)*log(2)-(2*nvar+m+1)*LogAbsDet(reshape(structpara(1:nvar*nvar),nvar,nvar));
                storevegfhZ(record,1) = LogVolumeElement(fs,structpara,r); 
                uw(record,1)          = exp(storevefh(record,1) - storevegfhZ(record,1));
                
            case 'irfs'
                
                irfpara                = fo_str2irfs(structpara);
                storevephi(record,1)   = LogVolumeElement(fo,structpara)   + LogVolumeElement(fo_str2irfs_inv,irfpara);
                storevegphiZ(record,1) = LogVolumeElement(fs,structpara,r) + LogVolumeElement(fo_str2irfs_inv,irfpara,r_irfs); 
                uw(record,1)           = exp(storevephi(record,1) - storevegphiZ(record,1));
                
            otherwise
                
                uw(record,1) = 1;
                
        end
        
    else
        
        uw(record,1) = 0;
        
    end
    
    if counter==iter_show
        
        display(['Number of draws = ',num2str(record)])
        display(['Remaining draws = ',num2str(nd-(record))])
        display(['5. France'])
        counter =0;
        
    end
    counter = counter + 1;
    record=record+1;
end

tElapsed = toc(tStart);
imp_w  = uw/sum(uw);
ne = floor(1/sum(imp_w.^2));

% store draws
Ltilde        = zeros(horizon+1,nvar,nvar,ne); % define array to store IRF
A0tilde       = zeros(nvar,nvar,ne); % define array to store A0
Aplustilde    = zeros(m,nvar,ne); % define array to store Aplus
FEVD          = zeros(nvar,size(index,2),ne);                 % define array to store FEVD
hist_is_draws = zeros(ne,1);   % define array to store draws from importance sampler

for s=1:min(ne,maxdraws)
    
    % draw: B,Sigma,Q
    is_draw = randsample(1:size(imp_w,1),1,true,imp_w);
    hist_is_draws(s,1)=is_draw;

    Bdraw       = Bdraws{is_draw,1};
    Sigmadraw   = Sigmadraws{is_draw,1};
    Qdraw       = Qdraws{is_draw,1};
    
    % compute matrix F: useful for FEVD
    hSigmadraw = hh(Sigmadraw);
    A0         = hSigmadraw\e;
    Aplus      = Bdraw*A0;
    % Page 8 ARRW
    for l=1:nlag-1
        A{l} = Aplus((l-1)*nvar+1:l*nvar,1:end);
        F((l-1)*nvar+1:l*nvar,1:nvar)=A{l}/A0;
    end
    A{nlag} = Aplus((nlag-1)*nvar+1:nlag*nvar,1:end);
    F((nlag-1)*nvar+1:nlag*nvar,:)=[A{nlag}/A0 extraF];
    
    x=[reshape(Bdraw,m*nvar,1); reshape(Sigmadraw,nvar*nvar,1); Qdraw(:)];
    structpara = f_h_inv(x,info);
    
    LIRF =IRF_horizons(structpara, nvar, nlag, m, 0:horizon);
    
    for h=0:horizon
        Ltilde(h+1,:,:,s) =  LIRF(1+h*nvar:(h+1)*nvar,:);
    end
    
    for j=1:size(index,2)
    FEVD(:,j,s)         = variancedecomposition(F',J,Sigmadraw, hh(Sigmadraw)'*Qdraw(:,1),nvar,index(1,j));
    end
    
    A0tilde(:,:,s)    = reshape(structpara(1:nvar*nvar),nvar,nvar);
    Aplustilde(:,:,s) = reshape(structpara(nvar*nvar+1:end),m,nvar);
end

A0tilde    = A0tilde(:,:,1:s);
Aplustilde = Aplustilde(:,:,1:s);
Ltilde     = Ltilde(:,:,:,1:s);
FEVD       = FEVD(:,:,1:s);

cd results
savefile='results_fr.mat';
save(savefile,'Ltilde','A0tilde','Aplustilde','imp_w','ne','FEVD');
cd ..





%% Germany %%
%==========================================================================
% housekeeping
%==========================================================================
clearvars -except usmpshock* 
close all;
userpath('clear');


rng('default'); % reinitialize the random number generator to its startup configuration
rng(0);         % set seed

currdir=pwd;
cd ..
get_help_dir_currdir=pwd;
addpath([get_help_dir_currdir,'/helpfunctions']); % set path to helper functions
cd(currdir)

%==========================================================================
% load the data and priors
%==========================================================================
data  = readtable([get_help_dir_currdir,'/data/csvfiles/dataset.csv']);
data   = data(find(strcmp(data.dates,'1970-01-01')==1):find(strcmp(data.dates,'2007-06-01')==1),:);
[Germany,~,~] = xlsread('Germany.xls','Sheet1','B122:H571');
exc_ge = Germany(:,1);     % exchange rate
ip_ge  = Germany(:,2);     % Un
cpi_ge = Germany(:,3);     % CPI
ir_ge  = Germany(:,4);     % short term interest rate
tb_ge = Germany(:,7);     % exports
[us,~,~] = xlsread('us.xls','dataset','B62:I511');
ip_us = 100*log(us(:,7));
cpi_us= 100*log(us(:,8));
cpr_us= 100*log(us(:,3));
tr_us= 100*log(us(:,4));
bog_us= 100*log(us(:,5));
ffr= us(:,6);
num  = [ip_us cpi_us cpr_us tr_us bog_us ffr exc_ge ip_ge cpi_ge ir_ge tb_ge];

%==========================================================================
% model setup
%==========================================================================
nlag      = 2;                  % number of lags
nvar      = 11;                   % number of endogenous variables
nex       = 0;                   % set equal to 1 if a constant is included; 0 otherwise
m         = nvar*nlag + nex;     % number of exogenous variables
nd        = 500000;                % number of orthogonal-reduced-form (B,Sigma,Q) draws
iter_show = 5e2;                 % display iteration every iter_show draws
horizon   = 60;                  % maximum horizon for IRFs
index     = 40;                  % define  horizons for the FEVD
horizons  = 0:5;                 % horizons to restrict
NS        = 1 + numel(horizons); % number of objects in F(A_{0},A_{+}) to which we impose sign and zero restrictios: F(THETA)=[A_{0};L_{0},...,L_{horizons}]
e         = eye(nvar);           % create identity matrix
maxdraws  = 1e4;                 % max number of importance sampling draws
conjugate = 'structural';        % structural or irfs or empty
%==========================================================================
% identification: declare Ss and Zs matrices
%==========================================================================
% restrictions on A0  and/or IRFs
% sign restrictions
S = cell(nvar,1);
for ii=1:nvar
    S{ii}=zeros(0,nvar*NS);
end

S{1}(1,nvar+2) = -1;S{1}(5,2*nvar+2)  = -1;S{1}(9,3*nvar+2)  = -1;S{1}(13,4*nvar+2) = -1;S{1}(17,5*nvar+2) = -1;S{1}(21,6*nvar+2) = -1;
S{1}(2,nvar+3) = -1;S{1}(6,2*nvar+3)  = -1;S{1}(10,3*nvar+3) = -1;S{1}(14,4*nvar+3) = -1;S{1}(18,5*nvar+3) = -1;S{1}(22,6*nvar+3) = -1;
S{1}(3,nvar+5) = -1;S{1}(7,2*nvar+5)  = -1;S{1}(11,3*nvar+5) = -1;S{1}(15,4*nvar+5) = -1;S{1}(19,5*nvar+5) = -1;S{1}(23,6*nvar+5) = -1;
S{1}(4,nvar+6) =  1;S{1}(8,2*nvar+6)  =  1;S{1}(12,3*nvar+6) =  1;S{1}(16,4*nvar+6) =  1;S{1}(20,5*nvar+6) =  1;S{1}(24,6*nvar+6) =  1;
S{1}(25,1)        = -1;
S{1}(26,2)        = -1;
S{1}(27,3)        = -1;
S{1}(28,6)        =  1;

% zero restrictions
Z=cell(nvar,1);
for i=1:nvar
    Z{i}=zeros(0,nvar*NS);
end
Z{1}(1,4)=1;
Z{1}(2,5)=1;
Z{1}(3,7)=1; 
Z{1}(4,8)=1; 
Z{1}(5,9)=1; 
Z{1}(6,10)=1;    Z{1}(7,11)=1; 

%==========================================================================
% Setup info
%==========================================================================
info=SetupInfo(nvar,m,Z,@(x)chol(x));

% ZF(A_{0},A_{+})
info.nlag     = nlag;
info.horizons = horizons;
info.ZF       = @(x,y)ZF(x,y);

% functions useful to compute the importance sampler weights
iw_info = info;
fs      = @(x)ff_h(x,iw_info);
r       = @(x)ZeroRestrictions(x,iw_info);

if strcmp(conjugate,'irfs')==1
    fo              = @(x)f_h(x,iw_info);
    fo_str2irfs     = @(x)StructuralToIRF(x,iw_info);
    fo_str2irfs_inv = @(x)IRFToStructural(x,iw_info);
    r_irfs          = @(x)IRFRestrictions_more_general(x,iw_info); 
end

% function useful to check the sign restrictions
fh_S_restrictions  = @(x)SF(x,iw_info,S);

%==========================================================================
% write data in Rubio, Waggoner, and Zha (RES 2010)'s notation
%==========================================================================
% yt(t) A0 = xt(t) Aplus + constant + et(t) for t=1...,T;
% yt(t)    = xt(t) B     + ut(t)            for t=1...,T;
% x(t)     = [yt(t-1), ... , yt(t-nlag), constant];
% matrix notation yt = xt*B + ut;
% xt=[yt_{-1} ones(T,1)];
yt = num(nlag+1:end,:);
T  = size(yt,1);
xt = zeros(T,nvar*nlag+nex);
for i=1:nlag
    xt(:,nvar*(i-1)+1:nvar*i) = num((nlag-(i-1)):end-i,:) ;
end
if nex>=1
    xt(:,nvar*nlag+nex)=ones(T,1);
end
% write data in Zellner (1971, pp 224-227) notation
Y = yt; % T by nvar matrix of observations
X = xt; % T by (nvar*nlag+1) matrix of regressors

% prior for reduced-form parameters
nnuBar              = 0;
OomegaBarInverse    = zeros(m);
PpsiBar             = zeros(m,nvar);
PphiBar             = zeros(nvar);

% posterior for reduced-form parameters
nnuTilde            = T +nnuBar;
OomegaTilde         = (X'*X  + OomegaBarInverse)\eye(m);
OomegaTildeInverse  =  X'*X  + OomegaBarInverse;
PpsiTilde           = OomegaTilde*(X'*Y + OomegaBarInverse*PpsiBar);
PphiTilde           = Y'*Y + PphiBar + PpsiBar'*OomegaBarInverse*PpsiBar - PpsiTilde'*OomegaTildeInverse*PpsiTilde;
PphiTilde           = (PphiTilde'+PphiTilde)*0.5;

% useful definitions
% definitios used to store orthogonal-reduced-form draws, volume elements, and unnormalized weights
Bdraws         = cell([nd,1]); % reduced-form lag parameters
Sigmadraws     = cell([nd,1]); % reduced-form covariance matrices
Qdraws         = cell([nd,1]); % orthogonal matrices
storevefh      = zeros(nd,1);  % volume element f_{h}
storevegfhZ    = zeros(nd,1);  % volume element g o f_{h}|Z
uw             = zeros(nd,1);  % unnormalized importance sampler weights

if strcmp(conjugate,'irfs')==1
    storevephi      = zeros(nd,1);  % volume element f_{h}
    storevegphiZ    = zeros(nd,1);  % volume element g o f_{h}|Z
end

% definitions related to IRFs; based on page 12 of Rubio, Waggoner, and Zha (RES 2010)
J      = [e;repmat(zeros(nvar),nlag-1,1)];
A      = cell(nlag,1);
extraF = repmat(zeros(nvar),1,nlag-1);
F      = zeros(nlag*nvar,nlag*nvar);
for l=1:nlag-1
    F((l-1)*nvar+1:l*nvar,nvar+1:nlag*nvar)=[repmat(zeros(nvar),1,l-1) e repmat(zeros(nvar),1,nlag-(l+1))];
end

% definition to facilitate the draws from B|Sigma
hh              = info.h;
cholOomegaTilde = hh(OomegaTilde)'; % this matrix is used to draw B|Sigma below

% initialize counters to track the state of the computations
counter = 1;
record  = 1;
count   = 0;
tStart = tic;
while record<=nd
    
    % step 1 in Algorithm 2
    Sigmadraw     = iwishrnd(PphiTilde,nnuTilde);
    cholSigmadraw = hh(Sigmadraw)';
    Bdraw         = kron(cholSigmadraw,cholOomegaTilde)*randn(m*nvar,1) + reshape(PpsiTilde,nvar*m,1);
    Bdraw         = reshape(Bdraw,nvar*nlag+nex,nvar);
    % store reduced-form draws
    Bdraws{record,1}     = Bdraw;
    Sigmadraws{record,1} = Sigmadraw;
    
    % steps 2:4 of Algorithm 2
    w           = DrawW(iw_info);   
    x           = [vec(Bdraw); vec(Sigmadraw); w];
    structpara  = ff_h_inv(x,iw_info);
    
    % store the matrix Q associated with step 3
    Qdraw            = SpheresToQ(w,iw_info,Bdraw,Sigmadraw);
    Qdraws{record,1} = reshape(Qdraw,nvar,nvar);
    
    % check if sign restrictions hold
    signs      = fh_S_restrictions(structpara);
    
    if (sum(signs{1}*e(:,1)>0))==size(signs{1}*e(:,1),1)
        
        count=count+1;
        
        % compute importance sampling weights
       
        switch conjugate
            
            case 'structural'
                
                storevefh(record,1)   = (nvar*(nvar+1)/2)*log(2)-(2*nvar+m+1)*LogAbsDet(reshape(structpara(1:nvar*nvar),nvar,nvar));
                storevegfhZ(record,1) = LogVolumeElement(fs,structpara,r); 
                uw(record,1)          = exp(storevefh(record,1) - storevegfhZ(record,1));
                
            case 'irfs'
                
                irfpara                = fo_str2irfs(structpara);
                storevephi(record,1)   = LogVolumeElement(fo,structpara)   + LogVolumeElement(fo_str2irfs_inv,irfpara);
                storevegphiZ(record,1) = LogVolumeElement(fs,structpara,r) + LogVolumeElement(fo_str2irfs_inv,irfpara,r_irfs); 
                uw(record,1)           = exp(storevephi(record,1) - storevegphiZ(record,1));
                
            otherwise
                
                uw(record,1) = 1;
                
        end
        
    else
        
        uw(record,1) = 0;
        
    end
    
    if counter==iter_show
        
        display(['Number of draws = ',num2str(record)])
        display(['Remaining draws = ',num2str(nd-(record))])
        display(['6. Germany'])
        counter =0;
        
    end
    counter = counter + 1;
    record=record+1;
end

tElapsed = toc(tStart);
imp_w  = uw/sum(uw);
ne = floor(1/sum(imp_w.^2));

% store draws
Ltilde        = zeros(horizon+1,nvar,nvar,ne); % define array to store IRF
A0tilde       = zeros(nvar,nvar,ne); % define array to store A0
Aplustilde    = zeros(m,nvar,ne); % define array to store Aplus
FEVD          = zeros(nvar,size(index,2),ne);                 % define array to store FEVD
hist_is_draws = zeros(ne,1);   % define array to store draws from importance sampler

for s=1:min(ne,maxdraws)
    
    % draw: B,Sigma,Q
    is_draw = randsample(1:size(imp_w,1),1,true,imp_w);
    hist_is_draws(s,1)=is_draw;

    Bdraw       = Bdraws{is_draw,1};
    Sigmadraw   = Sigmadraws{is_draw,1};
    Qdraw       = Qdraws{is_draw,1};
    
    % compute matrix F: useful for FEVD
    hSigmadraw = hh(Sigmadraw);
    A0         = hSigmadraw\e;
    Aplus      = Bdraw*A0;
    % Page 8 ARRW
    for l=1:nlag-1
        A{l} = Aplus((l-1)*nvar+1:l*nvar,1:end);
        F((l-1)*nvar+1:l*nvar,1:nvar)=A{l}/A0;
    end
    A{nlag} = Aplus((nlag-1)*nvar+1:nlag*nvar,1:end);
    F((nlag-1)*nvar+1:nlag*nvar,:)=[A{nlag}/A0 extraF];
    
    x=[reshape(Bdraw,m*nvar,1); reshape(Sigmadraw,nvar*nvar,1); Qdraw(:)];
    structpara = f_h_inv(x,info);
    
    LIRF =IRF_horizons(structpara, nvar, nlag, m, 0:horizon);
    
    for h=0:horizon
        Ltilde(h+1,:,:,s) =  LIRF(1+h*nvar:(h+1)*nvar,:);
    end
    
    for j=1:size(index,2)
    FEVD(:,j,s)         = variancedecomposition(F',J,Sigmadraw, hh(Sigmadraw)'*Qdraw(:,1),nvar,index(1,j));
    end
    
    A0tilde(:,:,s)    = reshape(structpara(1:nvar*nvar),nvar,nvar);
    Aplustilde(:,:,s) = reshape(structpara(nvar*nvar+1:end),m,nvar);
end

A0tilde    = A0tilde(:,:,1:s);
Aplustilde = Aplustilde(:,:,1:s);
Ltilde     = Ltilde(:,:,:,1:s);
FEVD       = FEVD(:,:,1:s);

cd results
savefile='results_ge.mat';
save(savefile,'Ltilde','A0tilde','Aplustilde','imp_w','ne','FEVD');
cd ..



%% Italy %%
%==========================================================================
% housekeeping
%==========================================================================
clear
close all;
userpath('clear');


rng('default'); % reinitialize the random number generator to its startup configuration
rng(0);         % set seed

currdir=pwd;
cd ..
get_help_dir_currdir=pwd;
addpath([get_help_dir_currdir,'/helpfunctions']); % set path to helper functions
cd(currdir)

%==========================================================================
% load the data and priors
%==========================================================================
data  = readtable([get_help_dir_currdir,'/data/csvfiles/dataset.csv']);
data   = data(find(strcmp(data.dates,'1970-01-01')==1):find(strcmp(data.dates,'2007-06-01')==1),:);
[Austria,~,~] = xlsread('Italy.xls','Sheet1','B122:H571');
exc_au = Austria(:,1);     
ip_au  = Austria(:,2);    
cpi_au = Austria(:,3);     
ir_au  = Austria(:,4);     
tb_au = Austria(:,7);     
[us,~,~] = xlsread('us.xls','dataset','B62:I511');
ip_us = 100*log(us(:,7));
cpi_us= 100*log(us(:,8));
cpr_us= 100*log(us(:,3));
tr_us= 100*log(us(:,4));
bog_us= 100*log(us(:,5));
ffr= us(:,6);

num  = [ip_us cpi_us cpr_us tr_us bog_us ffr exc_au ip_au cpi_au ir_au tb_au];

%==========================================================================
% model setup
%==========================================================================
nlag      = 3;                  % number of lags, HQ
nvar      = 11;                   % number of endogenous variables
nex       = 0;                   % set equal to 1 if a constant is included; 0 otherwise
m         = nvar*nlag + nex;     % number of exogenous variables
nd        = 500000;                % number of orthogonal-reduced-form (B,Sigma,Q) draws
iter_show = 5e2;                 % display iteration every iter_show draws
horizon   = 60;                  % maximum horizon for IRFs
index     = 40;                  % define  horizons for the FEVD
horizons  = 0:5;                 % horizons to restrict
NS        = 1 + numel(horizons); % number of objects in F(A_{0},A_{+}) to which we impose sign and zero restrictios: F(THETA)=[A_{0};L_{0},...,L_{horizons}]
e         = eye(nvar);           % create identity matrix
maxdraws  = 1e4;                 % max number of importance sampling draws
conjugate = 'structural';        % structural or irfs or empty
%==========================================================================
% identification: declare Ss and Zs matrices
%==========================================================================
% restrictions on A0  and/or IRFs
% sign restrictions
S = cell(nvar,1);
for ii=1:nvar
    S{ii}=zeros(0,nvar*NS);
end


S{1}(1,nvar+2) = -1;S{1}(5,2*nvar+2)  = -1;S{1}(9,3*nvar+2)  = -1;S{1}(13,4*nvar+2) = -1;S{1}(17,5*nvar+2) = -1;S{1}(21,6*nvar+2) = -1;
S{1}(2,nvar+3) = -1;S{1}(6,2*nvar+3)  = -1;S{1}(10,3*nvar+3) = -1;S{1}(14,4*nvar+3) = -1;S{1}(18,5*nvar+3) = -1;S{1}(22,6*nvar+3) = -1;
S{1}(3,nvar+5) = -1;S{1}(7,2*nvar+5)  = -1;S{1}(11,3*nvar+5) = -1;S{1}(15,4*nvar+5) = -1;S{1}(19,5*nvar+5) = -1;S{1}(23,6*nvar+5) = -1;
S{1}(4,nvar+6) =  1;S{1}(8,2*nvar+6)  =  1;S{1}(12,3*nvar+6) =  1;S{1}(16,4*nvar+6) =  1;S{1}(20,5*nvar+6) =  1;S{1}(24,6*nvar+6) =  1;
S{1}(25,1)        = -1;
S{1}(26,2)        = -1;
S{1}(27,3)        = -1;
S{1}(28,6)        =  1;

% zero restrictions
Z=cell(nvar,1);
for i=1:nvar
    Z{i}=zeros(0,nvar*NS);
end
Z{1}(1,4)=1;
Z{1}(2,5)=1;
Z{1}(3,7)=1; 
Z{1}(4,8)=1; 
Z{1}(5,9)=1; 
Z{1}(6,10)=1;    Z{1}(7,11)=1; 

%==========================================================================
% Setup info
%==========================================================================
info=SetupInfo(nvar,m,Z,@(x)chol(x));

% ZF(A_{0},A_{+})
info.nlag     = nlag;
info.horizons = horizons;
info.ZF       = @(x,y)ZF(x,y);

% functions useful to compute the importance sampler weights
iw_info = info;
fs      = @(x)ff_h(x,iw_info);
r       = @(x)ZeroRestrictions(x,iw_info);

if strcmp(conjugate,'irfs')==1
    fo              = @(x)f_h(x,iw_info);
    fo_str2irfs     = @(x)StructuralToIRF(x,iw_info);
    fo_str2irfs_inv = @(x)IRFToStructural(x,iw_info);
    r_irfs          = @(x)IRFRestrictions_more_general(x,iw_info); 
end

% function useful to check the sign restrictions
fh_S_restrictions  = @(x)SF(x,iw_info,S);

%==========================================================================
% write data in Rubio, Waggoner, and Zha (RES 2010)'s notation
%==========================================================================
% yt(t) A0 = xt(t) Aplus + constant + et(t) for t=1...,T;
% yt(t)    = xt(t) B     + ut(t)            for t=1...,T;
% x(t)     = [yt(t-1), ... , yt(t-nlag), constant];
% matrix notation yt = xt*B + ut;
% xt=[yt_{-1} ones(T,1)];
yt = num(nlag+1:end,:);
T  = size(yt,1);
xt = zeros(T,nvar*nlag+nex);
for i=1:nlag
    xt(:,nvar*(i-1)+1:nvar*i) = num((nlag-(i-1)):end-i,:) ;
end
if nex>=1
    xt(:,nvar*nlag+nex)=ones(T,1);
end
% write data in Zellner (1971, pp 224-227) notation
Y = yt; % T by nvar matrix of observations
X = xt; % T by (nvar*nlag+1) matrix of regressors

% prior for reduced-form parameters
nnuBar              = 0;
OomegaBarInverse    = zeros(m);
PpsiBar             = zeros(m,nvar);
PphiBar             = zeros(nvar);

% posterior for reduced-form parameters
nnuTilde            = T +nnuBar;
OomegaTilde         = (X'*X  + OomegaBarInverse)\eye(m);
OomegaTildeInverse  =  X'*X  + OomegaBarInverse;
PpsiTilde           = OomegaTilde*(X'*Y + OomegaBarInverse*PpsiBar);
PphiTilde           = Y'*Y + PphiBar + PpsiBar'*OomegaBarInverse*PpsiBar - PpsiTilde'*OomegaTildeInverse*PpsiTilde;
PphiTilde           = (PphiTilde'+PphiTilde)*0.5;

% useful definitions
% definitios used to store orthogonal-reduced-form draws, volume elements, and unnormalized weights
Bdraws         = cell([nd,1]); % reduced-form lag parameters
Sigmadraws     = cell([nd,1]); % reduced-form covariance matrices
Qdraws         = cell([nd,1]); % orthogonal matrices
storevefh      = zeros(nd,1);  % volume element f_{h}
storevegfhZ    = zeros(nd,1);  % volume element g o f_{h}|Z
uw             = zeros(nd,1);  % unnormalized importance sampler weights

if strcmp(conjugate,'irfs')==1
    storevephi      = zeros(nd,1);  % volume element f_{h}
    storevegphiZ    = zeros(nd,1);  % volume element g o f_{h}|Z
end

% definitions related to IRFs; based on page 12 of Rubio, Waggoner, and Zha (RES 2010)
J      = [e;repmat(zeros(nvar),nlag-1,1)];
A      = cell(nlag,1);
extraF = repmat(zeros(nvar),1,nlag-1);
F      = zeros(nlag*nvar,nlag*nvar);
for l=1:nlag-1
    F((l-1)*nvar+1:l*nvar,nvar+1:nlag*nvar)=[repmat(zeros(nvar),1,l-1) e repmat(zeros(nvar),1,nlag-(l+1))];
end

% definition to facilitate the draws from B|Sigma
hh              = info.h;
cholOomegaTilde = hh(OomegaTilde)'; % this matrix is used to draw B|Sigma below

% initialize counters to track the state of the computations
counter = 1;
record  = 1;
count   = 0;
tStart = tic;
while record<=nd
    
    % step 1 in Algorithm 2
    Sigmadraw     = iwishrnd(PphiTilde,nnuTilde);
    cholSigmadraw = hh(Sigmadraw)';
    Bdraw         = kron(cholSigmadraw,cholOomegaTilde)*randn(m*nvar,1) + reshape(PpsiTilde,nvar*m,1);
    Bdraw         = reshape(Bdraw,nvar*nlag+nex,nvar);
    % store reduced-form draws
    Bdraws{record,1}     = Bdraw;
    Sigmadraws{record,1} = Sigmadraw;
    
    % steps 2:4 of Algorithm 2
    w           = DrawW(iw_info);   
    x           = [vec(Bdraw); vec(Sigmadraw); w];
    structpara  = ff_h_inv(x,iw_info);
    
    % store the matrix Q associated with step 3
    Qdraw            = SpheresToQ(w,iw_info,Bdraw,Sigmadraw);
    Qdraws{record,1} = reshape(Qdraw,nvar,nvar);
    
    % check if sign restrictions hold
    signs      = fh_S_restrictions(structpara);
    
    if (sum(signs{1}*e(:,1)>0))==size(signs{1}*e(:,1),1)
        
        count=count+1;
        
        % compute importance sampling weights
       
        switch conjugate
            
            case 'structural'
                
                storevefh(record,1)   = (nvar*(nvar+1)/2)*log(2)-(2*nvar+m+1)*LogAbsDet(reshape(structpara(1:nvar*nvar),nvar,nvar));
                storevegfhZ(record,1) = LogVolumeElement(fs,structpara,r); 
                uw(record,1)          = exp(storevefh(record,1) - storevegfhZ(record,1));
                
            case 'irfs'
                
                irfpara                = fo_str2irfs(structpara);
                storevephi(record,1)   = LogVolumeElement(fo,structpara)   + LogVolumeElement(fo_str2irfs_inv,irfpara);
                storevegphiZ(record,1) = LogVolumeElement(fs,structpara,r) + LogVolumeElement(fo_str2irfs_inv,irfpara,r_irfs); 
                uw(record,1)           = exp(storevephi(record,1) - storevegphiZ(record,1));
                
            otherwise
                
                uw(record,1) = 1;
                
        end
        
    else
        
        uw(record,1) = 0;
        
    end
    
    if counter==iter_show
        
        display(['Number of draws = ',num2str(record)])
        display(['Remaining draws = ',num2str(nd-(record))])
        display(['7. Italy'])
        counter =0;
        
    end
    counter = counter + 1;
    record=record+1;
end

tElapsed = toc(tStart);
imp_w  = uw/sum(uw);
ne = floor(1/sum(imp_w.^2));

% store draws
Ltilde        = zeros(horizon+1,nvar,nvar,ne); % define array to store IRF
A0tilde       = zeros(nvar,nvar,ne); % define array to store A0
Aplustilde    = zeros(m,nvar,ne); % define array to store Aplus
FEVD          = zeros(nvar,size(index,2),ne);                 % define array to store FEVD
hist_is_draws = zeros(ne,1);   % define array to store draws from importance sampler

for s=1:min(ne,maxdraws)
    
    % draw: B,Sigma,Q
    is_draw = randsample(1:size(imp_w,1),1,true,imp_w);
    hist_is_draws(s,1)=is_draw;

    Bdraw       = Bdraws{is_draw,1};
    Sigmadraw   = Sigmadraws{is_draw,1};
    Qdraw       = Qdraws{is_draw,1};
    
    % compute matrix F: useful for FEVD
    hSigmadraw = hh(Sigmadraw);
    A0         = hSigmadraw\e;
    Aplus      = Bdraw*A0;
    % Page 8 ARRW
    for l=1:nlag-1
        A{l} = Aplus((l-1)*nvar+1:l*nvar,1:end);
        F((l-1)*nvar+1:l*nvar,1:nvar)=A{l}/A0;
    end
    A{nlag} = Aplus((nlag-1)*nvar+1:nlag*nvar,1:end);
    F((nlag-1)*nvar+1:nlag*nvar,:)=[A{nlag}/A0 extraF];
    
    x=[reshape(Bdraw,m*nvar,1); reshape(Sigmadraw,nvar*nvar,1); Qdraw(:)];
    structpara = f_h_inv(x,info);
    
    LIRF =IRF_horizons(structpara, nvar, nlag, m, 0:horizon);
    
    for h=0:horizon
        Ltilde(h+1,:,:,s) =  LIRF(1+h*nvar:(h+1)*nvar,:);
    end
    
    for j=1:size(index,2)
    FEVD(:,j,s)         = variancedecomposition(F',J,Sigmadraw, hh(Sigmadraw)'*Qdraw(:,1),nvar,index(1,j));
    end
    
    A0tilde(:,:,s)    = reshape(structpara(1:nvar*nvar),nvar,nvar);
    Aplustilde(:,:,s) = reshape(structpara(nvar*nvar+1:end),m,nvar);
end

A0tilde    = A0tilde(:,:,1:s);
Aplustilde = Aplustilde(:,:,1:s);
Ltilde     = Ltilde(:,:,:,1:s);
FEVD       = FEVD(:,:,1:s);



cd results
savefile='results_it.mat';
save(savefile,'Ltilde','A0tilde','Aplustilde','imp_w','ne','FEVD');
cd ..





%% JAPAN %%%
%==========================================================================
% housekeeping
%==========================================================================
clearvars -except usmpshock* 
close all;
userpath('clear');


rng('default'); % reinitialize the random number generator to its startup configuration
rng(0);         % set seed

currdir=pwd;
cd ..
get_help_dir_currdir=pwd;
addpath([get_help_dir_currdir,'/helpfunctions']); % set path to helper functions
cd(currdir)

%==========================================================================
% load the data and priors
%==========================================================================
data  = readtable([get_help_dir_currdir,'/data/csvfiles/dataset.csv']);
data   = data(find(strcmp(data.dates,'1970-01-01')==1):find(strcmp(data.dates,'2007-06-01')==1),:);
[Japan,~,~] = xlsread('Japan.xls','Sheet1','B122:H571');
exc_jp = Japan(:,1);     % exchange rate
ip_jp  = Japan(:,2);     % IP
cpi_jp = Japan(:,3);     % CPI
ir_jp  = Japan(:,4);     % short term interest rate
tb_jp = Japan(:,7);     % exports
[us,~,~] = xlsread('us.xls','dataset','B62:I511');
ip_us = 100*log(us(:,7));
cpi_us= 100*log(us(:,8));
cpr_us= 100*log(us(:,3));
tr_us= 100*log(us(:,4));
bog_us= 100*log(us(:,5));
ffr= us(:,6);
num  = [ip_us cpi_us cpr_us tr_us bog_us ffr exc_jp ip_jp cpi_jp ir_jp tb_jp];

%==========================================================================
% model setup
%==========================================================================
nlag      = 2;                  % number of lags
nvar      = 11;                   % number of endogenous variables
nex       = 0;                   % set equal to 1 if a constant is included; 0 otherwise
m         = nvar*nlag + nex;     % number of exogenous variables
nd        = 500000;                % number of orthogonal-reduced-form (B,Sigma,Q) draws
iter_show = 5e2;                 % display iteration every iter_show draws
horizon   = 60;                  % maximum horizon for IRFs
index     = 40;                  % define  horizons for the FEVD
horizons  = 0:5;                 % horizons to restrict
NS        = 1 + numel(horizons); % number of objects in F(A_{0},A_{+}) to which we impose sign and zero restrictios: F(THETA)=[A_{0};L_{0},...,L_{horizons}]
e         = eye(nvar);           % create identity matrix
maxdraws  = 1e4;                 % max number of importance sampling draws
conjugate = 'structural';        % structural or irfs or empty
%==========================================================================
% identification: declare Ss and Zs matrices
%==========================================================================
% restrictions on A0  and/or IRFs
% sign restrictions
S = cell(nvar,1);
for ii=1:nvar
    S{ii}=zeros(0,nvar*NS);
end

S{1}(1,nvar+2) = -1;S{1}(5,2*nvar+2)  = -1;S{1}(9,3*nvar+2)  = -1;S{1}(13,4*nvar+2) = -1;S{1}(17,5*nvar+2) = -1;S{1}(21,6*nvar+2) = -1;
S{1}(2,nvar+3) = -1;S{1}(6,2*nvar+3)  = -1;S{1}(10,3*nvar+3) = -1;S{1}(14,4*nvar+3) = -1;S{1}(18,5*nvar+3) = -1;S{1}(22,6*nvar+3) = -1;
S{1}(3,nvar+5) = -1;S{1}(7,2*nvar+5)  = -1;S{1}(11,3*nvar+5) = -1;S{1}(15,4*nvar+5) = -1;S{1}(19,5*nvar+5) = -1;S{1}(23,6*nvar+5) = -1;
S{1}(4,nvar+6) =  1;S{1}(8,2*nvar+6)  =  1;S{1}(12,3*nvar+6) =  1;S{1}(16,4*nvar+6) =  1;S{1}(20,5*nvar+6) =  1;S{1}(24,6*nvar+6) =  1;
S{1}(25,1)        = -1;
S{1}(26,2)        = -1;
S{1}(27,3)        = -1;
S{1}(28,6)        =  1;

% zero restrictions
Z=cell(nvar,1);
for i=1:nvar
    Z{i}=zeros(0,nvar*NS);
end
Z{1}(1,4)=1;
Z{1}(2,5)=1;
Z{1}(3,7)=1; 
Z{1}(4,8)=1; 
Z{1}(5,9)=1; 
Z{1}(6,10)=1;    Z{1}(7,11)=1; 

%==========================================================================
% Setup info
%==========================================================================
info=SetupInfo(nvar,m,Z,@(x)chol(x));

% ZF(A_{0},A_{+})
info.nlag     = nlag;
info.horizons = horizons;
info.ZF       = @(x,y)ZF(x,y);

% functions useful to compute the importance sampler weights
iw_info = info;
fs      = @(x)ff_h(x,iw_info);
r       = @(x)ZeroRestrictions(x,iw_info);

if strcmp(conjugate,'irfs')==1
    fo              = @(x)f_h(x,iw_info);
    fo_str2irfs     = @(x)StructuralToIRF(x,iw_info);
    fo_str2irfs_inv = @(x)IRFToStructural(x,iw_info);
    r_irfs          = @(x)IRFRestrictions_more_general(x,iw_info); 
end

% function useful to check the sign restrictions
fh_S_restrictions  = @(x)SF(x,iw_info,S);

%==========================================================================
% write data in Rubio, Waggoner, and Zha (RES 2010)'s notation
%==========================================================================
% yt(t) A0 = xt(t) Aplus + constant + et(t) for t=1...,T;
% yt(t)    = xt(t) B     + ut(t)            for t=1...,T;
% x(t)     = [yt(t-1), ... , yt(t-nlag), constant];
% matrix notation yt = xt*B + ut;
% xt=[yt_{-1} ones(T,1)];
yt = num(nlag+1:end,:);
T  = size(yt,1);
xt = zeros(T,nvar*nlag+nex);
for i=1:nlag
    xt(:,nvar*(i-1)+1:nvar*i) = num((nlag-(i-1)):end-i,:) ;
end
if nex>=1
    xt(:,nvar*nlag+nex)=ones(T,1);
end
% write data in Zellner (1971, pp 224-227) notation
Y = yt; % T by nvar matrix of observations
X = xt; % T by (nvar*nlag+1) matrix of regressors

% prior for reduced-form parameters
nnuBar              = 0;
OomegaBarInverse    = zeros(m);
PpsiBar             = zeros(m,nvar);
PphiBar             = zeros(nvar);

% posterior for reduced-form parameters
nnuTilde            = T +nnuBar;
OomegaTilde         = (X'*X  + OomegaBarInverse)\eye(m);
OomegaTildeInverse  =  X'*X  + OomegaBarInverse;
PpsiTilde           = OomegaTilde*(X'*Y + OomegaBarInverse*PpsiBar);
PphiTilde           = Y'*Y + PphiBar + PpsiBar'*OomegaBarInverse*PpsiBar - PpsiTilde'*OomegaTildeInverse*PpsiTilde;
PphiTilde           = (PphiTilde'+PphiTilde)*0.5;

% useful definitions
% definitios used to store orthogonal-reduced-form draws, volume elements, and unnormalized weights
Bdraws         = cell([nd,1]); % reduced-form lag parameters
Sigmadraws     = cell([nd,1]); % reduced-form covariance matrices
Qdraws         = cell([nd,1]); % orthogonal matrices
storevefh      = zeros(nd,1);  % volume element f_{h}
storevegfhZ    = zeros(nd,1);  % volume element g o f_{h}|Z
uw             = zeros(nd,1);  % unnormalized importance sampler weights

if strcmp(conjugate,'irfs')==1
    storevephi      = zeros(nd,1);  % volume element f_{h}
    storevegphiZ    = zeros(nd,1);  % volume element g o f_{h}|Z
end

% definitions related to IRFs; based on page 12 of Rubio, Waggoner, and Zha (RES 2010)
J      = [e;repmat(zeros(nvar),nlag-1,1)];
A      = cell(nlag,1);
extraF = repmat(zeros(nvar),1,nlag-1);
F      = zeros(nlag*nvar,nlag*nvar);
for l=1:nlag-1
    F((l-1)*nvar+1:l*nvar,nvar+1:nlag*nvar)=[repmat(zeros(nvar),1,l-1) e repmat(zeros(nvar),1,nlag-(l+1))];
end

% definition to facilitate the draws from B|Sigma
hh              = info.h;
cholOomegaTilde = hh(OomegaTilde)'; % this matrix is used to draw B|Sigma below

% initialize counters to track the state of the computations
counter = 1;
record  = 1;
count   = 0;
tStart = tic;
while record<=nd
    
    % step 1 in Algorithm 2
    Sigmadraw     = iwishrnd(PphiTilde,nnuTilde);
    cholSigmadraw = hh(Sigmadraw)';
    Bdraw         = kron(cholSigmadraw,cholOomegaTilde)*randn(m*nvar,1) + reshape(PpsiTilde,nvar*m,1);
    Bdraw         = reshape(Bdraw,nvar*nlag+nex,nvar);
    % store reduced-form draws
    Bdraws{record,1}     = Bdraw;
    Sigmadraws{record,1} = Sigmadraw;
    
    % steps 2:4 of Algorithm 2
    w           = DrawW(iw_info);   
    x           = [vec(Bdraw); vec(Sigmadraw); w];
    structpara  = ff_h_inv(x,iw_info);
    
    % store the matrix Q associated with step 3
    Qdraw            = SpheresToQ(w,iw_info,Bdraw,Sigmadraw);
    Qdraws{record,1} = reshape(Qdraw,nvar,nvar);
    
    % check if sign restrictions hold
    signs      = fh_S_restrictions(structpara);
    
    if (sum(signs{1}*e(:,1)>0))==size(signs{1}*e(:,1),1)
        
        count=count+1;
        
        % compute importance sampling weights
       
        switch conjugate
            
            case 'structural'
                
                storevefh(record,1)   = (nvar*(nvar+1)/2)*log(2)-(2*nvar+m+1)*LogAbsDet(reshape(structpara(1:nvar*nvar),nvar,nvar));
                storevegfhZ(record,1) = LogVolumeElement(fs,structpara,r); 
                uw(record,1)          = exp(storevefh(record,1) - storevegfhZ(record,1));
                
            case 'irfs'
                
                irfpara                = fo_str2irfs(structpara);
                storevephi(record,1)   = LogVolumeElement(fo,structpara)   + LogVolumeElement(fo_str2irfs_inv,irfpara);
                storevegphiZ(record,1) = LogVolumeElement(fs,structpara,r) + LogVolumeElement(fo_str2irfs_inv,irfpara,r_irfs); 
                uw(record,1)           = exp(storevephi(record,1) - storevegphiZ(record,1));
                
            otherwise
                
                uw(record,1) = 1;
                
        end
        
    else
        
        uw(record,1) = 0;
        
    end
    
    if counter==iter_show
        
        display(['Number of draws = ',num2str(record)])
        display(['Remaining draws = ',num2str(nd-(record))])
        display(['8. JAPAN'])
        counter =0;
        
    end
    counter = counter + 1;
    record=record+1;
end

tElapsed = toc(tStart);
imp_w  = uw/sum(uw);
ne = floor(1/sum(imp_w.^2));

% store draws
Ltilde        = zeros(horizon+1,nvar,nvar,ne); % define array to store IRF
A0tilde       = zeros(nvar,nvar,ne); % define array to store A0
Aplustilde    = zeros(m,nvar,ne); % define array to store Aplus
FEVD          = zeros(nvar,size(index,2),ne);                 % define array to store FEVD
hist_is_draws = zeros(ne,1);   % define array to store draws from importance sampler

for s=1:min(ne,maxdraws)
    
    % draw: B,Sigma,Q
    is_draw = randsample(1:size(imp_w,1),1,true,imp_w);
    hist_is_draws(s,1)=is_draw;

    Bdraw       = Bdraws{is_draw,1};
    Sigmadraw   = Sigmadraws{is_draw,1};
    Qdraw       = Qdraws{is_draw,1};
    
    % compute matrix F: useful for FEVD
    hSigmadraw = hh(Sigmadraw);
    A0         = hSigmadraw\e;
    Aplus      = Bdraw*A0;
    % Page 8 ARRW
    for l=1:nlag-1
        A{l} = Aplus((l-1)*nvar+1:l*nvar,1:end);
        F((l-1)*nvar+1:l*nvar,1:nvar)=A{l}/A0;
    end
    A{nlag} = Aplus((nlag-1)*nvar+1:nlag*nvar,1:end);
    F((nlag-1)*nvar+1:nlag*nvar,:)=[A{nlag}/A0 extraF];
    
    x=[reshape(Bdraw,m*nvar,1); reshape(Sigmadraw,nvar*nvar,1); Qdraw(:)];
    structpara = f_h_inv(x,info);
    
    LIRF =IRF_horizons(structpara, nvar, nlag, m, 0:horizon);
    
    for h=0:horizon
        Ltilde(h+1,:,:,s) =  LIRF(1+h*nvar:(h+1)*nvar,:);
    end
    
    for j=1:size(index,2)
    FEVD(:,j,s)         = variancedecomposition(F',J,Sigmadraw, hh(Sigmadraw)'*Qdraw(:,1),nvar,index(1,j));
    end
    
    A0tilde(:,:,s)    = reshape(structpara(1:nvar*nvar),nvar,nvar);
    Aplustilde(:,:,s) = reshape(structpara(nvar*nvar+1:end),m,nvar);
end

A0tilde    = A0tilde(:,:,1:s);
Aplustilde = Aplustilde(:,:,1:s);
Ltilde     = Ltilde(:,:,:,1:s);
FEVD       = FEVD(:,:,1:s);

cd results
savefile='results_jp.mat';
save(savefile,'Ltilde','A0tilde','Aplustilde','imp_w','ne','FEVD');
cd ..




%% Netherlands %%
%==========================================================================
% housekeeping
%==========================================================================
clearvars -except usmpshock* 
close all;
userpath('clear');


rng('default'); % reinitialize the random number generator to its startup configuration
rng(0);         % set seed

currdir=pwd;
cd ..
get_help_dir_currdir=pwd;
addpath([get_help_dir_currdir,'/helpfunctions']); % set path to helper functions
cd(currdir)

%==========================================================================
% load the data and priors
%==========================================================================
data  = readtable([get_help_dir_currdir,'/data/csvfiles/dataset.csv']);
data   = data(find(strcmp(data.dates,'1974-01-01')==1):find(strcmp(data.dates,'2007-06-01')==1),:);
[Netherland,~,~] = xlsread('Netherland.xls','Sheet1','B170:H571');
exc_ne = Netherland(:,1);     % exchange rate
ip_ne  = Netherland(:,2);     % Un
cpi_ne = Netherland(:,3);     % CPI
ir_ne  = Netherland(:,4);     % short term interest rate
tb_ne = Netherland(:,7);     % tb
[us,~,~] = xlsread('us.xls','dataset','B110:I511');
ip_us = 100*log(us(:,7));
cpi_us= 100*log(us(:,8));
cpr_us= 100*log(us(:,3));
tr_us= 100*log(us(:,4));
bog_us= 100*log(us(:,5));
ffr= us(:,6);
num  = [ip_us cpi_us cpr_us tr_us bog_us ffr exc_ne ip_ne cpi_ne ir_ne tb_ne];

%==========================================================================
% model setup
%==========================================================================
nlag      = 2;                  % number of lags
nvar      = 11;                   % number of endogenous variables
nex       = 0;                   % set equal to 1 if a constant is included; 0 otherwise
m         = nvar*nlag + nex;     % number of exogenous variables
nd        = 500000;                % number of orthogonal-reduced-form (B,Sigma,Q) draws
iter_show = 5e2;                 % display iteration every iter_show draws
horizon   = 60;                  % maximum horizon for IRFs
index     = 40;                  % define  horizons for the FEVD
horizons  = 0:5;                 % horizons to restrict
NS        = 1 + numel(horizons); % number of objects in F(A_{0},A_{+}) to which we impose sign and zero restrictios: F(THETA)=[A_{0};L_{0},...,L_{horizons}]
e         = eye(nvar);           % create identity matrix
maxdraws  = 1e4;                 % max number of importance sampling draws
conjugate = 'structural';        % structural or irfs or empty
%==========================================================================
% identification: declare Ss and Zs matrices
%==========================================================================
% restrictions on A0  and/or IRFs
% sign restrictions
S = cell(nvar,1);
for ii=1:nvar
    S{ii}=zeros(0,nvar*NS);
end

S{1}(1,nvar+2) = -1;S{1}(5,2*nvar+2)  = -1;S{1}(9,3*nvar+2)  = -1;S{1}(13,4*nvar+2) = -1;S{1}(17,5*nvar+2) = -1;S{1}(21,6*nvar+2) = -1;
S{1}(2,nvar+3) = -1;S{1}(6,2*nvar+3)  = -1;S{1}(10,3*nvar+3) = -1;S{1}(14,4*nvar+3) = -1;S{1}(18,5*nvar+3) = -1;S{1}(22,6*nvar+3) = -1;
S{1}(3,nvar+5) = -1;S{1}(7,2*nvar+5)  = -1;S{1}(11,3*nvar+5) = -1;S{1}(15,4*nvar+5) = -1;S{1}(19,5*nvar+5) = -1;S{1}(23,6*nvar+5) = -1;
S{1}(4,nvar+6) =  1;S{1}(8,2*nvar+6)  =  1;S{1}(12,3*nvar+6) =  1;S{1}(16,4*nvar+6) =  1;S{1}(20,5*nvar+6) =  1;S{1}(24,6*nvar+6) =  1;
S{1}(25,1)        = -1;
S{1}(26,2)        = -1;
S{1}(27,3)        = -1;
S{1}(28,6)        =  1;

% zero restrictions
Z=cell(nvar,1);
for i=1:nvar
    Z{i}=zeros(0,nvar*NS);
end
Z{1}(1,4)=1;
Z{1}(2,5)=1;
Z{1}(3,7)=1; 
Z{1}(4,8)=1; 
Z{1}(5,9)=1; 
Z{1}(6,10)=1;    Z{1}(7,11)=1; 

%==========================================================================
% Setup info
%==========================================================================
info=SetupInfo(nvar,m,Z,@(x)chol(x));

% ZF(A_{0},A_{+})
info.nlag     = nlag;
info.horizons = horizons;
info.ZF       = @(x,y)ZF(x,y);

% functions useful to compute the importance sampler weights
iw_info = info;
fs      = @(x)ff_h(x,iw_info);
r       = @(x)ZeroRestrictions(x,iw_info);

if strcmp(conjugate,'irfs')==1
    fo              = @(x)f_h(x,iw_info);
    fo_str2irfs     = @(x)StructuralToIRF(x,iw_info);
    fo_str2irfs_inv = @(x)IRFToStructural(x,iw_info);
    r_irfs          = @(x)IRFRestrictions_more_general(x,iw_info); 
end

% function useful to check the sign restrictions
fh_S_restrictions  = @(x)SF(x,iw_info,S);

%==========================================================================
% write data in Rubio, Waggoner, and Zha (RES 2010)'s notation
%==========================================================================
% yt(t) A0 = xt(t) Aplus + constant + et(t) for t=1...,T;
% yt(t)    = xt(t) B     + ut(t)            for t=1...,T;
% x(t)     = [yt(t-1), ... , yt(t-nlag), constant];
% matrix notation yt = xt*B + ut;
% xt=[yt_{-1} ones(T,1)];
yt = num(nlag+1:end,:);
T  = size(yt,1);
xt = zeros(T,nvar*nlag+nex);
for i=1:nlag
    xt(:,nvar*(i-1)+1:nvar*i) = num((nlag-(i-1)):end-i,:) ;
end
if nex>=1
    xt(:,nvar*nlag+nex)=ones(T,1);
end
% write data in Zellner (1971, pp 224-227) notation
Y = yt; % T by nvar matrix of observations
X = xt; % T by (nvar*nlag+1) matrix of regressors

% prior for reduced-form parameters
nnuBar              = 0;
OomegaBarInverse    = zeros(m);
PpsiBar             = zeros(m,nvar);
PphiBar             = zeros(nvar);

% posterior for reduced-form parameters
nnuTilde            = T +nnuBar;
OomegaTilde         = (X'*X  + OomegaBarInverse)\eye(m);
OomegaTildeInverse  =  X'*X  + OomegaBarInverse;
PpsiTilde           = OomegaTilde*(X'*Y + OomegaBarInverse*PpsiBar);
PphiTilde           = Y'*Y + PphiBar + PpsiBar'*OomegaBarInverse*PpsiBar - PpsiTilde'*OomegaTildeInverse*PpsiTilde;
PphiTilde           = (PphiTilde'+PphiTilde)*0.5;

% useful definitions
% definitios used to store orthogonal-reduced-form draws, volume elements, and unnormalized weights
Bdraws         = cell([nd,1]); % reduced-form lag parameters
Sigmadraws     = cell([nd,1]); % reduced-form covariance matrices
Qdraws         = cell([nd,1]); % orthogonal matrices
storevefh      = zeros(nd,1);  % volume element f_{h}
storevegfhZ    = zeros(nd,1);  % volume element g o f_{h}|Z
uw             = zeros(nd,1);  % unnormalized importance sampler weights

if strcmp(conjugate,'irfs')==1
    storevephi      = zeros(nd,1);  % volume element f_{h}
    storevegphiZ    = zeros(nd,1);  % volume element g o f_{h}|Z
end

% definitions related to IRFs; based on page 12 of Rubio, Waggoner, and Zha (RES 2010)
J      = [e;repmat(zeros(nvar),nlag-1,1)];
A      = cell(nlag,1);
extraF = repmat(zeros(nvar),1,nlag-1);
F      = zeros(nlag*nvar,nlag*nvar);
for l=1:nlag-1
    F((l-1)*nvar+1:l*nvar,nvar+1:nlag*nvar)=[repmat(zeros(nvar),1,l-1) e repmat(zeros(nvar),1,nlag-(l+1))];
end

% definition to facilitate the draws from B|Sigma
hh              = info.h;
cholOomegaTilde = hh(OomegaTilde)'; % this matrix is used to draw B|Sigma below

% initialize counters to track the state of the computations
counter = 1;
record  = 1;
count   = 0;
tStart = tic;
while record<=nd
    
    % step 1 in Algorithm 2
    Sigmadraw     = iwishrnd(PphiTilde,nnuTilde);
    cholSigmadraw = hh(Sigmadraw)';
    Bdraw         = kron(cholSigmadraw,cholOomegaTilde)*randn(m*nvar,1) + reshape(PpsiTilde,nvar*m,1);
    Bdraw         = reshape(Bdraw,nvar*nlag+nex,nvar);
    % store reduced-form draws
    Bdraws{record,1}     = Bdraw;
    Sigmadraws{record,1} = Sigmadraw;
    
    % steps 2:4 of Algorithm 2
    w           = DrawW(iw_info);   
    x           = [vec(Bdraw); vec(Sigmadraw); w];
    structpara  = ff_h_inv(x,iw_info);
    
    % store the matrix Q associated with step 3
    Qdraw            = SpheresToQ(w,iw_info,Bdraw,Sigmadraw);
    Qdraws{record,1} = reshape(Qdraw,nvar,nvar);
    
    % check if sign restrictions hold
    signs      = fh_S_restrictions(structpara);
    
    if (sum(signs{1}*e(:,1)>0))==size(signs{1}*e(:,1),1)
        
        count=count+1;
        
        % compute importance sampling weights
       
        switch conjugate
            
            case 'structural'
                
                storevefh(record,1)   = (nvar*(nvar+1)/2)*log(2)-(2*nvar+m+1)*LogAbsDet(reshape(structpara(1:nvar*nvar),nvar,nvar));
                storevegfhZ(record,1) = LogVolumeElement(fs,structpara,r); 
                uw(record,1)          = exp(storevefh(record,1) - storevegfhZ(record,1));
                
            case 'irfs'
                
                irfpara                = fo_str2irfs(structpara);
                storevephi(record,1)   = LogVolumeElement(fo,structpara)   + LogVolumeElement(fo_str2irfs_inv,irfpara);
                storevegphiZ(record,1) = LogVolumeElement(fs,structpara,r) + LogVolumeElement(fo_str2irfs_inv,irfpara,r_irfs); 
                uw(record,1)           = exp(storevephi(record,1) - storevegphiZ(record,1));
                
            otherwise
                
                uw(record,1) = 1;
                
        end
        
    else
        
        uw(record,1) = 0;
        
    end
    
    if counter==iter_show
        
        display(['Number of draws = ',num2str(record)])
        display(['Remaining draws = ',num2str(nd-(record))])
        display(['9. Netherlands'])
        counter =0;
        
    end
    counter = counter + 1;
    record=record+1;
end

tElapsed = toc(tStart);
imp_w  = uw/sum(uw);
ne = floor(1/sum(imp_w.^2));

% store draws
Ltilde        = zeros(horizon+1,nvar,nvar,ne); % define array to store IRF
A0tilde       = zeros(nvar,nvar,ne); % define array to store A0
Aplustilde    = zeros(m,nvar,ne); % define array to store Aplus
FEVD          = zeros(nvar,size(index,2),ne);                 % define array to store FEVD
hist_is_draws = zeros(ne,1);   % define array to store draws from importance sampler

for s=1:min(ne,maxdraws)
    
    % draw: B,Sigma,Q
    is_draw = randsample(1:size(imp_w,1),1,true,imp_w);
    hist_is_draws(s,1)=is_draw;

    Bdraw       = Bdraws{is_draw,1};
    Sigmadraw   = Sigmadraws{is_draw,1};
    Qdraw       = Qdraws{is_draw,1};
    
    % compute matrix F: useful for FEVD
    hSigmadraw = hh(Sigmadraw);
    A0         = hSigmadraw\e;
    Aplus      = Bdraw*A0;
    % Page 8 ARRW
    for l=1:nlag-1
        A{l} = Aplus((l-1)*nvar+1:l*nvar,1:end);
        F((l-1)*nvar+1:l*nvar,1:nvar)=A{l}/A0;
    end
    A{nlag} = Aplus((nlag-1)*nvar+1:nlag*nvar,1:end);
    F((nlag-1)*nvar+1:nlag*nvar,:)=[A{nlag}/A0 extraF];
    
    x=[reshape(Bdraw,m*nvar,1); reshape(Sigmadraw,nvar*nvar,1); Qdraw(:)];
    structpara = f_h_inv(x,info);
    
    LIRF =IRF_horizons(structpara, nvar, nlag, m, 0:horizon);
    
    for h=0:horizon
        Ltilde(h+1,:,:,s) =  LIRF(1+h*nvar:(h+1)*nvar,:);
    end
    
    for j=1:size(index,2)
    FEVD(:,j,s)         = variancedecomposition(F',J,Sigmadraw, hh(Sigmadraw)'*Qdraw(:,1),nvar,index(1,j));
    end
    
    A0tilde(:,:,s)    = reshape(structpara(1:nvar*nvar),nvar,nvar);
    Aplustilde(:,:,s) = reshape(structpara(nvar*nvar+1:end),m,nvar);
end

A0tilde    = A0tilde(:,:,1:s);
Aplustilde = Aplustilde(:,:,1:s);
Ltilde     = Ltilde(:,:,:,1:s);
FEVD       = FEVD(:,:,1:s);

cd results
savefile='results_ne.mat';
save(savefile,'Ltilde','A0tilde','Aplustilde','imp_w','ne','FEVD');
cd ..




%% Spain %%%
%==========================================================================
% housekeeping
%==========================================================================
clearvars -except usmpshock* 
close all;
userpath('clear');


rng('default'); % reinitialize the random number generator to its startup configuration
rng(0);         % set seed

currdir=pwd;
cd ..
get_help_dir_currdir=pwd;
addpath([get_help_dir_currdir,'/helpfunctions']); % set path to helper functions
cd(currdir)

%==========================================================================
% load the data and priors
%==========================================================================
data  = readtable([get_help_dir_currdir,'/data/csvfiles/dataset.csv']);
data   = data(find(strcmp(data.dates,'1977-01-01')==1):find(strcmp(data.dates,'2007-06-01')==1),:);
[Spain,~,~] = xlsread('Spain.xls','Sheet1','B206:H571');
exc_sp = Spain(:,1);     % exchange rate
ip_sp  = Spain(:,2);     % IP
cpi_sp = Spain(:,3);     % CPI
ir_sp  = Spain(:,4);     % short term interest rate
tb_sp  = Spain(:,7);     %
[us,~,~] = xlsread('us.xls','dataset','B146:I511');
ip_us = 100*log(us(:,7));
cpi_us= 100*log(us(:,8));
cpr_us= 100*log(us(:,3));
tr_us= 100*log(us(:,4));
bog_us= 100*log(us(:,5));
ffr= us(:,6);
num  = [ip_us cpi_us cpr_us tr_us bog_us ffr exc_sp ip_sp cpi_sp ir_sp tb_sp];

%==========================================================================
% model setup
%==========================================================================
nlag      = 3;                  % number of lags
nvar      = 11;                   % number of endogenous variables
nex       = 0;                   % set equal to 1 if a constant is included; 0 otherwise
m         = nvar*nlag + nex;     % number of exogenous variables
nd        = 500000;                % number of orthogonal-reduced-form (B,Sigma,Q) draws
iter_show = 5e2;                 % display iteration every iter_show draws
horizon   = 60;                  % maximum horizon for IRFs
index     = 40;                  % define  horizons for the FEVD
horizons  = 0:5;                 % horizons to restrict
NS        = 1 + numel(horizons); % number of objects in F(A_{0},A_{+}) to which we impose sign and zero restrictios: F(THETA)=[A_{0};L_{0},...,L_{horizons}]
e         = eye(nvar);           % create identity matrix
maxdraws  = 1e4;                 % max number of importance sampling draws
conjugate = 'structural';        % structural or irfs or empty
%==========================================================================
% identification: declare Ss and Zs matrices
%==========================================================================
% restrictions on A0  and/or IRFs
% sign restrictions
S = cell(nvar,1);
for ii=1:nvar
    S{ii}=zeros(0,nvar*NS);
end

S{1}(1,nvar+2) = -1;S{1}(5,2*nvar+2)  = -1;S{1}(9,3*nvar+2)  = -1;S{1}(13,4*nvar+2) = -1;S{1}(17,5*nvar+2) = -1;S{1}(21,6*nvar+2) = -1;
S{1}(2,nvar+3) = -1;S{1}(6,2*nvar+3)  = -1;S{1}(10,3*nvar+3) = -1;S{1}(14,4*nvar+3) = -1;S{1}(18,5*nvar+3) = -1;S{1}(22,6*nvar+3) = -1;
S{1}(3,nvar+5) = -1;S{1}(7,2*nvar+5)  = -1;S{1}(11,3*nvar+5) = -1;S{1}(15,4*nvar+5) = -1;S{1}(19,5*nvar+5) = -1;S{1}(23,6*nvar+5) = -1;
S{1}(4,nvar+6) =  1;S{1}(8,2*nvar+6)  =  1;S{1}(12,3*nvar+6) =  1;S{1}(16,4*nvar+6) =  1;S{1}(20,5*nvar+6) =  1;S{1}(24,6*nvar+6) =  1;
S{1}(25,1)        = -1;
S{1}(26,2)        = -1;
S{1}(27,3)        = -1;
S{1}(28,6)        =  1;

% zero restrictions
Z=cell(nvar,1);
for i=1:nvar
    Z{i}=zeros(0,nvar*NS);
end
Z{1}(1,4)=1;
Z{1}(2,5)=1;
Z{1}(3,7)=1; 
Z{1}(4,8)=1; 
Z{1}(5,9)=1; 
Z{1}(6,10)=1;    Z{1}(7,11)=1; 

%==========================================================================
% Setup info
%==========================================================================
info=SetupInfo(nvar,m,Z,@(x)chol(x));

% ZF(A_{0},A_{+})
info.nlag     = nlag;
info.horizons = horizons;
info.ZF       = @(x,y)ZF(x,y);

% functions useful to compute the importance sampler weights
iw_info = info;
fs      = @(x)ff_h(x,iw_info);
r       = @(x)ZeroRestrictions(x,iw_info);

if strcmp(conjugate,'irfs')==1
    fo              = @(x)f_h(x,iw_info);
    fo_str2irfs     = @(x)StructuralToIRF(x,iw_info);
    fo_str2irfs_inv = @(x)IRFToStructural(x,iw_info);
    r_irfs          = @(x)IRFRestrictions_more_general(x,iw_info); 
end

% function useful to check the sign restrictions
fh_S_restrictions  = @(x)SF(x,iw_info,S);

%==========================================================================
% write data in Rubio, Waggoner, and Zha (RES 2010)'s notation
%==========================================================================
% yt(t) A0 = xt(t) Aplus + constant + et(t) for t=1...,T;
% yt(t)    = xt(t) B     + ut(t)            for t=1...,T;
% x(t)     = [yt(t-1), ... , yt(t-nlag), constant];
% matrix notation yt = xt*B + ut;
% xt=[yt_{-1} ones(T,1)];
yt = num(nlag+1:end,:);
T  = size(yt,1);
xt = zeros(T,nvar*nlag+nex);
for i=1:nlag
    xt(:,nvar*(i-1)+1:nvar*i) = num((nlag-(i-1)):end-i,:) ;
end
if nex>=1
    xt(:,nvar*nlag+nex)=ones(T,1);
end
% write data in Zellner (1971, pp 224-227) notation
Y = yt; % T by nvar matrix of observations
X = xt; % T by (nvar*nlag+1) matrix of regressors

% prior for reduced-form parameters
nnuBar              = 0;
OomegaBarInverse    = zeros(m);
PpsiBar             = zeros(m,nvar);
PphiBar             = zeros(nvar);

% posterior for reduced-form parameters
nnuTilde            = T +nnuBar;
OomegaTilde         = (X'*X  + OomegaBarInverse)\eye(m);
OomegaTildeInverse  =  X'*X  + OomegaBarInverse;
PpsiTilde           = OomegaTilde*(X'*Y + OomegaBarInverse*PpsiBar);
PphiTilde           = Y'*Y + PphiBar + PpsiBar'*OomegaBarInverse*PpsiBar - PpsiTilde'*OomegaTildeInverse*PpsiTilde;
PphiTilde           = (PphiTilde'+PphiTilde)*0.5;

% useful definitions
% definitios used to store orthogonal-reduced-form draws, volume elements, and unnormalized weights
Bdraws         = cell([nd,1]); % reduced-form lag parameters
Sigmadraws     = cell([nd,1]); % reduced-form covariance matrices
Qdraws         = cell([nd,1]); % orthogonal matrices
storevefh      = zeros(nd,1);  % volume element f_{h}
storevegfhZ    = zeros(nd,1);  % volume element g o f_{h}|Z
uw             = zeros(nd,1);  % unnormalized importance sampler weights

if strcmp(conjugate,'irfs')==1
    storevephi      = zeros(nd,1);  % volume element f_{h}
    storevegphiZ    = zeros(nd,1);  % volume element g o f_{h}|Z
end

% definitions related to IRFs; based on page 12 of Rubio, Waggoner, and Zha (RES 2010)
J      = [e;repmat(zeros(nvar),nlag-1,1)];
A      = cell(nlag,1);
extraF = repmat(zeros(nvar),1,nlag-1);
F      = zeros(nlag*nvar,nlag*nvar);
for l=1:nlag-1
    F((l-1)*nvar+1:l*nvar,nvar+1:nlag*nvar)=[repmat(zeros(nvar),1,l-1) e repmat(zeros(nvar),1,nlag-(l+1))];
end

% definition to facilitate the draws from B|Sigma
hh              = info.h;
cholOomegaTilde = hh(OomegaTilde)'; % this matrix is used to draw B|Sigma below

% initialize counters to track the state of the computations
counter = 1;
record  = 1;
count   = 0;
tStart = tic;
while record<=nd
    
    % step 1 in Algorithm 2
    Sigmadraw     = iwishrnd(PphiTilde,nnuTilde);
    cholSigmadraw = hh(Sigmadraw)';
    Bdraw         = kron(cholSigmadraw,cholOomegaTilde)*randn(m*nvar,1) + reshape(PpsiTilde,nvar*m,1);
    Bdraw         = reshape(Bdraw,nvar*nlag+nex,nvar);
    % store reduced-form draws
    Bdraws{record,1}     = Bdraw;
    Sigmadraws{record,1} = Sigmadraw;
    
    % steps 2:4 of Algorithm 2
    w           = DrawW(iw_info);   
    x           = [vec(Bdraw); vec(Sigmadraw); w];
    structpara  = ff_h_inv(x,iw_info);
    
    % store the matrix Q associated with step 3
    Qdraw            = SpheresToQ(w,iw_info,Bdraw,Sigmadraw);
    Qdraws{record,1} = reshape(Qdraw,nvar,nvar);
    
    % check if sign restrictions hold
    signs      = fh_S_restrictions(structpara);
    
    if (sum(signs{1}*e(:,1)>0))==size(signs{1}*e(:,1),1)
        
        count=count+1;
        
        % compute importance sampling weights
       
        switch conjugate
            
            case 'structural'
                
                storevefh(record,1)   = (nvar*(nvar+1)/2)*log(2)-(2*nvar+m+1)*LogAbsDet(reshape(structpara(1:nvar*nvar),nvar,nvar));
                storevegfhZ(record,1) = LogVolumeElement(fs,structpara,r); 
                uw(record,1)          = exp(storevefh(record,1) - storevegfhZ(record,1));
                
            case 'irfs'
                
                irfpara                = fo_str2irfs(structpara);
                storevephi(record,1)   = LogVolumeElement(fo,structpara)   + LogVolumeElement(fo_str2irfs_inv,irfpara);
                storevegphiZ(record,1) = LogVolumeElement(fs,structpara,r) + LogVolumeElement(fo_str2irfs_inv,irfpara,r_irfs); 
                uw(record,1)           = exp(storevephi(record,1) - storevegphiZ(record,1));
                
            otherwise
                
                uw(record,1) = 1;
                
        end
        
    else
        
        uw(record,1) = 0;
        
    end
    
    if counter==iter_show
        
        display(['Number of draws = ',num2str(record)])
        display(['Remaining draws = ',num2str(nd-(record))])
        display(['10. Spain'])
        counter =0;
        
    end
    counter = counter + 1;
    record=record+1;
end

tElapsed = toc(tStart);
imp_w  = uw/sum(uw);
ne = floor(1/sum(imp_w.^2));

% store draws
Ltilde        = zeros(horizon+1,nvar,nvar,ne); % define array to store IRF
A0tilde       = zeros(nvar,nvar,ne); % define array to store A0
Aplustilde    = zeros(m,nvar,ne); % define array to store Aplus
FEVD          = zeros(nvar,size(index,2),ne);                 % define array to store FEVD
hist_is_draws = zeros(ne,1);   % define array to store draws from importance sampler

for s=1:min(ne,maxdraws)
    
    % draw: B,Sigma,Q
    is_draw = randsample(1:size(imp_w,1),1,true,imp_w);
    hist_is_draws(s,1)=is_draw;

    Bdraw       = Bdraws{is_draw,1};
    Sigmadraw   = Sigmadraws{is_draw,1};
    Qdraw       = Qdraws{is_draw,1};
    
    % compute matrix F: useful for FEVD
    hSigmadraw = hh(Sigmadraw);
    A0         = hSigmadraw\e;
    Aplus      = Bdraw*A0;
    % Page 8 ARRW
    for l=1:nlag-1
        A{l} = Aplus((l-1)*nvar+1:l*nvar,1:end);
        F((l-1)*nvar+1:l*nvar,1:nvar)=A{l}/A0;
    end
    A{nlag} = Aplus((nlag-1)*nvar+1:nlag*nvar,1:end);
    F((nlag-1)*nvar+1:nlag*nvar,:)=[A{nlag}/A0 extraF];
    
    x=[reshape(Bdraw,m*nvar,1); reshape(Sigmadraw,nvar*nvar,1); Qdraw(:)];
    structpara = f_h_inv(x,info);
    
    LIRF =IRF_horizons(structpara, nvar, nlag, m, 0:horizon);
    
    for h=0:horizon
        Ltilde(h+1,:,:,s) =  LIRF(1+h*nvar:(h+1)*nvar,:);
    end
    
    for j=1:size(index,2)
    FEVD(:,j,s)         = variancedecomposition(F',J,Sigmadraw, hh(Sigmadraw)'*Qdraw(:,1),nvar,index(1,j));
    end
    
    A0tilde(:,:,s)    = reshape(structpara(1:nvar*nvar),nvar,nvar);
    Aplustilde(:,:,s) = reshape(structpara(nvar*nvar+1:end),m,nvar);
end

A0tilde    = A0tilde(:,:,1:s);
Aplustilde = Aplustilde(:,:,1:s);
Ltilde     = Ltilde(:,:,:,1:s);
FEVD       = FEVD(:,:,1:s);

cd results
savefile='results_sp.mat';
save(savefile,'Ltilde','A0tilde','Aplustilde','imp_w','ne','FEVD');
cd ..





%% Sweden %%%
%==========================================================================
% housekeeping
%==========================================================================
clearvars -except usmpshock*
close all;
userpath('clear');


rng('default'); % reinitialize the random number generator to its startup configuration
rng(0);         % set seed

currdir=pwd;
cd ..
get_help_dir_currdir=pwd;
addpath([get_help_dir_currdir,'/helpfunctions']); % set path to helper functions
cd(currdir)

%==========================================================================
% load the data and priors
%==========================================================================
data  = readtable([get_help_dir_currdir,'/data/csvfiles/dataset.csv']);
data   = data(find(strcmp(data.dates,'1970-01-01')==1):find(strcmp(data.dates,'2007-06-01')==1),:);
[Sweden,~,~] = xlsread('Sweden.xls','Sheet1','B122:H571');
exc_sw = Sweden(:,1);     % exchange rate
ip_sw  = Sweden(:,2);     % IP
cpi_sw = Sweden(:,3);     % CPI
ir_sw  = Sweden(:,4);     % short term interest rate
tb_sw  = Sweden(:,7);     % tb
[us,~,~] = xlsread('us.xls','dataset','B62:I511');
ip_us = 100*log(us(:,7));
cpi_us= 100*log(us(:,8));
cpr_us= 100*log(us(:,3));
tr_us= 100*log(us(:,4));
bog_us= 100*log(us(:,5));
ffr= us(:,6);
num  = [ip_us cpi_us cpr_us tr_us bog_us ffr exc_sw ip_sw cpi_sw ir_sw tb_sw];

%==========================================================================
% model setup
%==========================================================================
nlag      = 2;                  % number of lags
nvar      = 11;                   % number of endogenous variables
nex       = 0;                   % set equal to 1 if a constant is included; 0 otherwise
m         = nvar*nlag + nex;     % number of exogenous variables
nd        = 500000;                % number of orthogonal-reduced-form (B,Sigma,Q) draws
iter_show = 5e2;                 % display iteration every iter_show draws
horizon   = 60;                  % maximum horizon for IRFs
index     = 40;                  % define  horizons for the FEVD
horizons  = 0:5;                 % horizons to restrict
NS        = 1 + numel(horizons); % number of objects in F(A_{0},A_{+}) to which we impose sign and zero restrictios: F(THETA)=[A_{0};L_{0},...,L_{horizons}]
e         = eye(nvar);           % create identity matrix
maxdraws  = 1e4;                 % max number of importance sampling draws
conjugate = 'structural';        % structural or irfs or empty
%==========================================================================
% identification: declare Ss and Zs matrices
%==========================================================================
% restrictions on A0  and/or IRFs
% sign restrictions
S = cell(nvar,1);
for ii=1:nvar
    S{ii}=zeros(0,nvar*NS);
end

S{1}(1,nvar+2) = -1;S{1}(5,2*nvar+2)  = -1;S{1}(9,3*nvar+2)  = -1;S{1}(13,4*nvar+2) = -1;S{1}(17,5*nvar+2) = -1;S{1}(21,6*nvar+2) = -1;
S{1}(2,nvar+3) = -1;S{1}(6,2*nvar+3)  = -1;S{1}(10,3*nvar+3) = -1;S{1}(14,4*nvar+3) = -1;S{1}(18,5*nvar+3) = -1;S{1}(22,6*nvar+3) = -1;
S{1}(3,nvar+5) = -1;S{1}(7,2*nvar+5)  = -1;S{1}(11,3*nvar+5) = -1;S{1}(15,4*nvar+5) = -1;S{1}(19,5*nvar+5) = -1;S{1}(23,6*nvar+5) = -1;
S{1}(4,nvar+6) =  1;S{1}(8,2*nvar+6)  =  1;S{1}(12,3*nvar+6) =  1;S{1}(16,4*nvar+6) =  1;S{1}(20,5*nvar+6) =  1;S{1}(24,6*nvar+6) =  1;
S{1}(25,1)        = -1;
S{1}(26,2)        = -1;
S{1}(27,3)        = -1;
S{1}(28,6)        =  1;

% zero restrictions
Z=cell(nvar,1);
for i=1:nvar
    Z{i}=zeros(0,nvar*NS);
end
Z{1}(1,4)=1;
Z{1}(2,5)=1;
Z{1}(3,7)=1; 
Z{1}(4,8)=1; 
Z{1}(5,9)=1; 
Z{1}(6,10)=1;    Z{1}(7,11)=1; 

%==========================================================================
% Setup info
%==========================================================================
info=SetupInfo(nvar,m,Z,@(x)chol(x));

% ZF(A_{0},A_{+})
info.nlag     = nlag;
info.horizons = horizons;
info.ZF       = @(x,y)ZF(x,y);

% functions useful to compute the importance sampler weights
iw_info = info;
fs      = @(x)ff_h(x,iw_info);
r       = @(x)ZeroRestrictions(x,iw_info);

if strcmp(conjugate,'irfs')==1
    fo              = @(x)f_h(x,iw_info);
    fo_str2irfs     = @(x)StructuralToIRF(x,iw_info);
    fo_str2irfs_inv = @(x)IRFToStructural(x,iw_info);
    r_irfs          = @(x)IRFRestrictions_more_general(x,iw_info); 
end

% function useful to check the sign restrictions
fh_S_restrictions  = @(x)SF(x,iw_info,S);

%==========================================================================
% write data in Rubio, Waggoner, and Zha (RES 2010)'s notation
%==========================================================================
% yt(t) A0 = xt(t) Aplus + constant + et(t) for t=1...,T;
% yt(t)    = xt(t) B     + ut(t)            for t=1...,T;
% x(t)     = [yt(t-1), ... , yt(t-nlag), constant];
% matrix notation yt = xt*B + ut;
% xt=[yt_{-1} ones(T,1)];
yt = num(nlag+1:end,:);
T  = size(yt,1);
xt = zeros(T,nvar*nlag+nex);
for i=1:nlag
    xt(:,nvar*(i-1)+1:nvar*i) = num((nlag-(i-1)):end-i,:) ;
end
if nex>=1
    xt(:,nvar*nlag+nex)=ones(T,1);
end
% write data in Zellner (1971, pp 224-227) notation
Y = yt; % T by nvar matrix of observations
X = xt; % T by (nvar*nlag+1) matrix of regressors

% prior for reduced-form parameters
nnuBar              = 0;
OomegaBarInverse    = zeros(m);
PpsiBar             = zeros(m,nvar);
PphiBar             = zeros(nvar);

% posterior for reduced-form parameters
nnuTilde            = T +nnuBar;
OomegaTilde         = (X'*X  + OomegaBarInverse)\eye(m);
OomegaTildeInverse  =  X'*X  + OomegaBarInverse;
PpsiTilde           = OomegaTilde*(X'*Y + OomegaBarInverse*PpsiBar);
PphiTilde           = Y'*Y + PphiBar + PpsiBar'*OomegaBarInverse*PpsiBar - PpsiTilde'*OomegaTildeInverse*PpsiTilde;
PphiTilde           = (PphiTilde'+PphiTilde)*0.5;

% useful definitions
% definitios used to store orthogonal-reduced-form draws, volume elements, and unnormalized weights
Bdraws         = cell([nd,1]); % reduced-form lag parameters
Sigmadraws     = cell([nd,1]); % reduced-form covariance matrices
Qdraws         = cell([nd,1]); % orthogonal matrices
storevefh      = zeros(nd,1);  % volume element f_{h}
storevegfhZ    = zeros(nd,1);  % volume element g o f_{h}|Z
uw             = zeros(nd,1);  % unnormalized importance sampler weights

if strcmp(conjugate,'irfs')==1
    storevephi      = zeros(nd,1);  % volume element f_{h}
    storevegphiZ    = zeros(nd,1);  % volume element g o f_{h}|Z
end

% definitions related to IRFs; based on page 12 of Rubio, Waggoner, and Zha (RES 2010)
J      = [e;repmat(zeros(nvar),nlag-1,1)];
A      = cell(nlag,1);
extraF = repmat(zeros(nvar),1,nlag-1);
F      = zeros(nlag*nvar,nlag*nvar);
for l=1:nlag-1
    F((l-1)*nvar+1:l*nvar,nvar+1:nlag*nvar)=[repmat(zeros(nvar),1,l-1) e repmat(zeros(nvar),1,nlag-(l+1))];
end

% definition to facilitate the draws from B|Sigma
hh              = info.h;
cholOomegaTilde = hh(OomegaTilde)'; % this matrix is used to draw B|Sigma below

% initialize counters to track the state of the computations
counter = 1;
record  = 1;
count   = 0;
tStart = tic;
while record<=nd
    
    % step 1 in Algorithm 2
    Sigmadraw     = iwishrnd(PphiTilde,nnuTilde);
    cholSigmadraw = hh(Sigmadraw)';
    Bdraw         = kron(cholSigmadraw,cholOomegaTilde)*randn(m*nvar,1) + reshape(PpsiTilde,nvar*m,1);
    Bdraw         = reshape(Bdraw,nvar*nlag+nex,nvar);
    % store reduced-form draws
    Bdraws{record,1}     = Bdraw;
    Sigmadraws{record,1} = Sigmadraw;
    
    % steps 2:4 of Algorithm 2
    w           = DrawW(iw_info);   
    x           = [vec(Bdraw); vec(Sigmadraw); w];
    structpara  = ff_h_inv(x,iw_info);
    
    % store the matrix Q associated with step 3
    Qdraw            = SpheresToQ(w,iw_info,Bdraw,Sigmadraw);
    Qdraws{record,1} = reshape(Qdraw,nvar,nvar);
    
    % check if sign restrictions hold
    signs      = fh_S_restrictions(structpara);
    
    if (sum(signs{1}*e(:,1)>0))==size(signs{1}*e(:,1),1)
        
        count=count+1;
        
        % compute importance sampling weights
       
        switch conjugate
            
            case 'structural'
                
                storevefh(record,1)   = (nvar*(nvar+1)/2)*log(2)-(2*nvar+m+1)*LogAbsDet(reshape(structpara(1:nvar*nvar),nvar,nvar));
                storevegfhZ(record,1) = LogVolumeElement(fs,structpara,r); 
                uw(record,1)          = exp(storevefh(record,1) - storevegfhZ(record,1));
                
            case 'irfs'
                
                irfpara                = fo_str2irfs(structpara);
                storevephi(record,1)   = LogVolumeElement(fo,structpara)   + LogVolumeElement(fo_str2irfs_inv,irfpara);
                storevegphiZ(record,1) = LogVolumeElement(fs,structpara,r) + LogVolumeElement(fo_str2irfs_inv,irfpara,r_irfs); 
                uw(record,1)           = exp(storevephi(record,1) - storevegphiZ(record,1));
                
            otherwise
                
                uw(record,1) = 1;
                
        end
        
    else
        
        uw(record,1) = 0;
        
    end
    
    if counter==iter_show
        
        display(['Number of draws = ',num2str(record)])
        display(['Remaining draws = ',num2str(nd-(record))])
        display(['11. Sweden'])
        counter =0;
        
    end
    counter = counter + 1;
    record=record+1;
end

tElapsed = toc(tStart);
imp_w  = uw/sum(uw);
ne = floor(1/sum(imp_w.^2));

% store draws
Ltilde        = zeros(horizon+1,nvar,nvar,ne); % define array to store IRF
A0tilde       = zeros(nvar,nvar,ne); % define array to store A0
Aplustilde    = zeros(m,nvar,ne); % define array to store Aplus
FEVD          = zeros(nvar,size(index,2),ne);                 % define array to store FEVD
hist_is_draws = zeros(ne,1);   % define array to store draws from importance sampler

for s=1:min(ne,maxdraws)
    
    % draw: B,Sigma,Q
    is_draw = randsample(1:size(imp_w,1),1,true,imp_w);
    hist_is_draws(s,1)=is_draw;

    Bdraw       = Bdraws{is_draw,1};
    Sigmadraw   = Sigmadraws{is_draw,1};
    Qdraw       = Qdraws{is_draw,1};
    
    % compute matrix F: useful for FEVD
    hSigmadraw = hh(Sigmadraw);
    A0         = hSigmadraw\e;
    Aplus      = Bdraw*A0;
    % Page 8 ARRW
    for l=1:nlag-1
        A{l} = Aplus((l-1)*nvar+1:l*nvar,1:end);
        F((l-1)*nvar+1:l*nvar,1:nvar)=A{l}/A0;
    end
    A{nlag} = Aplus((nlag-1)*nvar+1:nlag*nvar,1:end);
    F((nlag-1)*nvar+1:nlag*nvar,:)=[A{nlag}/A0 extraF];
    
    x=[reshape(Bdraw,m*nvar,1); reshape(Sigmadraw,nvar*nvar,1); Qdraw(:)];
    structpara = f_h_inv(x,info);
    
    LIRF =IRF_horizons(structpara, nvar, nlag, m, 0:horizon);
    
    for h=0:horizon
        Ltilde(h+1,:,:,s) =  LIRF(1+h*nvar:(h+1)*nvar,:);
    end
    
    for j=1:size(index,2)
    FEVD(:,j,s)         = variancedecomposition(F',J,Sigmadraw, hh(Sigmadraw)'*Qdraw(:,1),nvar,index(1,j));
    end
    
    A0tilde(:,:,s)    = reshape(structpara(1:nvar*nvar),nvar,nvar);
    Aplustilde(:,:,s) = reshape(structpara(nvar*nvar+1:end),m,nvar);
end

A0tilde    = A0tilde(:,:,1:s);
Aplustilde = Aplustilde(:,:,1:s);
Ltilde     = Ltilde(:,:,:,1:s);
FEVD       = FEVD(:,:,1:s);

cd results
savefile='results_sw.mat';
save(savefile,'Ltilde','A0tilde','Aplustilde','imp_w','ne','FEVD');
cd ..






%% TURKEY %%
%==========================================================================
% housekeeping
%==========================================================================
clearvars -except usmpshock*
close all;
userpath('clear');


rng('default'); % reinitialize the random number generator to its startup configuration
rng(0);         % set seed

currdir=pwd;
cd ..
get_help_dir_currdir=pwd;
addpath([get_help_dir_currdir,'/helpfunctions']); % set path to helper functions
cd(currdir)

%==========================================================================
% load the data and priors
%==========================================================================
data  = readtable([get_help_dir_currdir,'/data/csvfiles/dataset.csv']);
data   = data(find(strcmp(data.dates,'1970-01-01')==1):find(strcmp(data.dates,'2007-06-01')==1),:);
[Norway,~,~] = xlsread('Turkey.xls','Sheet1','B2:G451');
exc_no = Norway(:,1);     % exchange rate
ip_no  = Norway(:,2);     % IP
cpi_no = Norway(:,3);     % CPI
ir_no  = Norway(:,4);     % short term interest rate
tb_no  = Norway(:,5);     %
[us,~,~] = xlsread('us.xls','dataset','B62:I511');
ip_us = 100*log(us(:,7));
cpi_us= 100*log(us(:,8));
cpr_us= 100*log(us(:,3));
tr_us= 100*log(us(:,4));
bog_us= 100*log(us(:,5));
ffr= us(:,6);
num  = [ip_us cpi_us cpr_us tr_us bog_us ffr exc_no ip_no cpi_no ir_no tb_no];

%==========================================================================
% model setup
%==========================================================================
nlag      = 2;                  % number of lags
nvar      = 11;                   % number of endogenous variables
nex       = 0;                   % set equal to 1 if a constant is included; 0 otherwise
m         = nvar*nlag + nex;     % number of exogenous variables
nd        = 500000;                % number of orthogonal-reduced-form (B,Sigma,Q) draws
iter_show = 5e2;                 % display iteration every iter_show draws
horizon   = 60;                  % maximum horizon for IRFs
index     = 40;                  % define  horizons for the FEVD
horizons  = 0:5;                 % horizons to restrict
NS        = 1 + numel(horizons); % number of objects in F(A_{0},A_{+}) to which we impose sign and zero restrictios: F(THETA)=[A_{0};L_{0},...,L_{horizons}]
e         = eye(nvar);           % create identity matrix
maxdraws  = 1e4;                 % max number of importance sampling draws
conjugate = 'structural';        % structural or irfs or empty
%==========================================================================
% identification: declare Ss and Zs matrices
%==========================================================================
% restrictions on A0  and/or IRFs
% sign restrictions
S = cell(nvar,1);
for ii=1:nvar
    S{ii}=zeros(0,nvar*NS);
end

S{1}(1,nvar+2) = -1;S{1}(5,2*nvar+2)  = -1;S{1}(9,3*nvar+2)  = -1;S{1}(13,4*nvar+2) = -1;S{1}(17,5*nvar+2) = -1;S{1}(21,6*nvar+2) = -1;
S{1}(2,nvar+3) = -1;S{1}(6,2*nvar+3)  = -1;S{1}(10,3*nvar+3) = -1;S{1}(14,4*nvar+3) = -1;S{1}(18,5*nvar+3) = -1;S{1}(22,6*nvar+3) = -1;
S{1}(3,nvar+5) = -1;S{1}(7,2*nvar+5)  = -1;S{1}(11,3*nvar+5) = -1;S{1}(15,4*nvar+5) = -1;S{1}(19,5*nvar+5) = -1;S{1}(23,6*nvar+5) = -1;
S{1}(4,nvar+6) =  1;S{1}(8,2*nvar+6)  =  1;S{1}(12,3*nvar+6) =  1;S{1}(16,4*nvar+6) =  1;S{1}(20,5*nvar+6) =  1;S{1}(24,6*nvar+6) =  1;
S{1}(25,1)        = -1;
S{1}(26,2)        = -1;
S{1}(27,3)        = -1;
S{1}(28,6)        =  1;

% zero restrictions
Z=cell(nvar,1);
for i=1:nvar
    Z{i}=zeros(0,nvar*NS);
end
Z{1}(1,4)=1;
Z{1}(2,5)=1;
Z{1}(3,7)=1; 
Z{1}(4,8)=1; 
Z{1}(5,9)=1; 
Z{1}(6,10)=1;    Z{1}(7,11)=1; 

%==========================================================================
% Setup info
%==========================================================================
info=SetupInfo(nvar,m,Z,@(x)chol(x));

% ZF(A_{0},A_{+})
info.nlag     = nlag;
info.horizons = horizons;
info.ZF       = @(x,y)ZF(x,y);

% functions useful to compute the importance sampler weights
iw_info = info;
fs      = @(x)ff_h(x,iw_info);
r       = @(x)ZeroRestrictions(x,iw_info);

if strcmp(conjugate,'irfs')==1
    fo              = @(x)f_h(x,iw_info);
    fo_str2irfs     = @(x)StructuralToIRF(x,iw_info);
    fo_str2irfs_inv = @(x)IRFToStructural(x,iw_info);
    r_irfs          = @(x)IRFRestrictions_more_general(x,iw_info); 
end

% function useful to check the sign restrictions
fh_S_restrictions  = @(x)SF(x,iw_info,S);

%==========================================================================
% write data in Rubio, Waggoner, and Zha (RES 2010)'s notation
%==========================================================================
% yt(t) A0 = xt(t) Aplus + constant + et(t) for t=1...,T;
% yt(t)    = xt(t) B     + ut(t)            for t=1...,T;
% x(t)     = [yt(t-1), ... , yt(t-nlag), constant];
% matrix notation yt = xt*B + ut;
% xt=[yt_{-1} ones(T,1)];
yt = num(nlag+1:end,:);
T  = size(yt,1);
xt = zeros(T,nvar*nlag+nex);
for i=1:nlag
    xt(:,nvar*(i-1)+1:nvar*i) = num((nlag-(i-1)):end-i,:) ;
end
if nex>=1
    xt(:,nvar*nlag+nex)=ones(T,1);
end
% write data in Zellner (1971, pp 224-227) notation
Y = yt; % T by nvar matrix of observations
X = xt; % T by (nvar*nlag+1) matrix of regressors

% prior for reduced-form parameters
nnuBar              = 0;
OomegaBarInverse    = zeros(m);
PpsiBar             = zeros(m,nvar);
PphiBar             = zeros(nvar);

% posterior for reduced-form parameters
nnuTilde            = T +nnuBar;
OomegaTilde         = (X'*X  + OomegaBarInverse)\eye(m);
OomegaTildeInverse  =  X'*X  + OomegaBarInverse;
PpsiTilde           = OomegaTilde*(X'*Y + OomegaBarInverse*PpsiBar);
PphiTilde           = Y'*Y + PphiBar + PpsiBar'*OomegaBarInverse*PpsiBar - PpsiTilde'*OomegaTildeInverse*PpsiTilde;
PphiTilde           = (PphiTilde'+PphiTilde)*0.5;

% useful definitions
% definitios used to store orthogonal-reduced-form draws, volume elements, and unnormalized weights
Bdraws         = cell([nd,1]); % reduced-form lag parameters
Sigmadraws     = cell([nd,1]); % reduced-form covariance matrices
Qdraws         = cell([nd,1]); % orthogonal matrices
storevefh      = zeros(nd,1);  % volume element f_{h}
storevegfhZ    = zeros(nd,1);  % volume element g o f_{h}|Z
uw             = zeros(nd,1);  % unnormalized importance sampler weights

if strcmp(conjugate,'irfs')==1
    storevephi      = zeros(nd,1);  % volume element f_{h}
    storevegphiZ    = zeros(nd,1);  % volume element g o f_{h}|Z
end

% definitions related to IRFs; based on page 12 of Rubio, Waggoner, and Zha (RES 2010)
J      = [e;repmat(zeros(nvar),nlag-1,1)];
A      = cell(nlag,1);
extraF = repmat(zeros(nvar),1,nlag-1);
F      = zeros(nlag*nvar,nlag*nvar);
for l=1:nlag-1
    F((l-1)*nvar+1:l*nvar,nvar+1:nlag*nvar)=[repmat(zeros(nvar),1,l-1) e repmat(zeros(nvar),1,nlag-(l+1))];
end

% definition to facilitate the draws from B|Sigma
hh              = info.h;
cholOomegaTilde = hh(OomegaTilde)'; % this matrix is used to draw B|Sigma below

% initialize counters to track the state of the computations
counter = 1;
record  = 1;
count   = 0;
tStart = tic;
while record<=nd
    
    % step 1 in Algorithm 2
    Sigmadraw     = iwishrnd(PphiTilde,nnuTilde);
    cholSigmadraw = hh(Sigmadraw)';
    Bdraw         = kron(cholSigmadraw,cholOomegaTilde)*randn(m*nvar,1) + reshape(PpsiTilde,nvar*m,1);
    Bdraw         = reshape(Bdraw,nvar*nlag+nex,nvar);
    % store reduced-form draws
    Bdraws{record,1}     = Bdraw;
    Sigmadraws{record,1} = Sigmadraw;
    
    % steps 2:4 of Algorithm 2
    w           = DrawW(iw_info);   
    x           = [vec(Bdraw); vec(Sigmadraw); w];
    structpara  = ff_h_inv(x,iw_info);
    
    % store the matrix Q associated with step 3
    Qdraw            = SpheresToQ(w,iw_info,Bdraw,Sigmadraw);
    Qdraws{record,1} = reshape(Qdraw,nvar,nvar);
    
    % check if sign restrictions hold
    signs      = fh_S_restrictions(structpara);
    
    if (sum(signs{1}*e(:,1)>0))==size(signs{1}*e(:,1),1)
        
        count=count+1;
        
        % compute importance sampling weights
       
        switch conjugate
            
            case 'structural'
                
                storevefh(record,1)   = (nvar*(nvar+1)/2)*log(2)-(2*nvar+m+1)*LogAbsDet(reshape(structpara(1:nvar*nvar),nvar,nvar));
                storevegfhZ(record,1) = LogVolumeElement(fs,structpara,r); 
                uw(record,1)          = exp(storevefh(record,1) - storevegfhZ(record,1));
                
            case 'irfs'
                
                irfpara                = fo_str2irfs(structpara);
                storevephi(record,1)   = LogVolumeElement(fo,structpara)   + LogVolumeElement(fo_str2irfs_inv,irfpara);
                storevegphiZ(record,1) = LogVolumeElement(fs,structpara,r) + LogVolumeElement(fo_str2irfs_inv,irfpara,r_irfs); 
                uw(record,1)           = exp(storevephi(record,1) - storevegphiZ(record,1));
                
            otherwise
                
                uw(record,1) = 1;
                
        end
        
    else
        
        uw(record,1) = 0;
        
    end
    
    if counter==iter_show
        
        display(['Number of draws = ',num2str(record)])
        display(['Remaining draws = ',num2str(nd-(record))])
        display(['12. Turkey'])
        counter =0;
        
    end
    counter = counter + 1;
    record=record+1;
end

tElapsed = toc(tStart);
imp_w  = uw/sum(uw);
ne = floor(1/sum(imp_w.^2));

% store draws
Ltilde        = zeros(horizon+1,nvar,nvar,ne); % define array to store IRF
A0tilde       = zeros(nvar,nvar,ne); % define array to store A0
Aplustilde    = zeros(m,nvar,ne); % define array to store Aplus
FEVD          = zeros(nvar,size(index,2),ne);                 % define array to store FEVD
hist_is_draws = zeros(ne,1);   % define array to store draws from importance sampler

for s=1:min(ne,maxdraws)
    
    % draw: B,Sigma,Q
    is_draw = randsample(1:size(imp_w,1),1,true,imp_w);
    hist_is_draws(s,1)=is_draw;

    Bdraw       = Bdraws{is_draw,1};
    Sigmadraw   = Sigmadraws{is_draw,1};
    Qdraw       = Qdraws{is_draw,1};
    
    % compute matrix F: useful for FEVD
    hSigmadraw = hh(Sigmadraw);
    A0         = hSigmadraw\e;
    Aplus      = Bdraw*A0;
    % Page 8 ARRW
    for l=1:nlag-1
        A{l} = Aplus((l-1)*nvar+1:l*nvar,1:end);
        F((l-1)*nvar+1:l*nvar,1:nvar)=A{l}/A0;
    end
    A{nlag} = Aplus((nlag-1)*nvar+1:nlag*nvar,1:end);
    F((nlag-1)*nvar+1:nlag*nvar,:)=[A{nlag}/A0 extraF];
    
    x=[reshape(Bdraw,m*nvar,1); reshape(Sigmadraw,nvar*nvar,1); Qdraw(:)];
    structpara = f_h_inv(x,info);
    
    LIRF =IRF_horizons(structpara, nvar, nlag, m, 0:horizon);
    
    for h=0:horizon
        Ltilde(h+1,:,:,s) =  LIRF(1+h*nvar:(h+1)*nvar,:);
    end
    
    for j=1:size(index,2)
    FEVD(:,j,s)         = variancedecomposition(F',J,Sigmadraw, hh(Sigmadraw)'*Qdraw(:,1),nvar,index(1,j));
    end
    
    A0tilde(:,:,s)    = reshape(structpara(1:nvar*nvar),nvar,nvar);
    Aplustilde(:,:,s) = reshape(structpara(nvar*nvar+1:end),m,nvar);
end

A0tilde    = A0tilde(:,:,1:s);
Aplustilde = Aplustilde(:,:,1:s);
Ltilde     = Ltilde(:,:,:,1:s);
FEVD       = FEVD(:,:,1:s);

cd results
savefile='results_tu.mat';
save(savefile,'Ltilde','A0tilde','Aplustilde','imp_w','ne','FEVD');
cd ..




%% UK %%
%==========================================================================
% housekeeping
%==========================================================================
clearvars -except usmpshock* 
close all;
userpath('clear');


rng('default'); % reinitialize the random number generator to its startup configuration
rng(0);         % set seed

currdir=pwd;
cd ..
get_help_dir_currdir=pwd;
addpath([get_help_dir_currdir,'/helpfunctions']); % set path to helper functions
cd(currdir)

%==========================================================================
% load the data and priors
%==========================================================================
data  = readtable([get_help_dir_currdir,'/data/csvfiles/dataset.csv']);
data   = data(find(strcmp(data.dates,'1970-01-01')==1):find(strcmp(data.dates,'2007-06-01')==1),:);
[UK,~,~] = xlsread('UK.xls','Sheet1','B122:H571');
exc_uk = UK(:,1);     % exchange rate
ip_uk  = UK(:,2);     % IP
cpi_uk = UK(:,3);     % CPI
ir_uk  = UK(:,4);     % short term interest rate
tb_uk = UK(:,7);     % exports
[us,~,~] = xlsread('us.xls','dataset','B62:I511');
ip_us = 100*log(us(:,7));
cpi_us= 100*log(us(:,8));
cpr_us= 100*log(us(:,3));
tr_us= 100*log(us(:,4));
bog_us= 100*log(us(:,5));
ffr= us(:,6);
num  = [ip_us cpi_us cpr_us tr_us bog_us ffr exc_uk ip_uk cpi_uk ir_uk tb_uk];

%==========================================================================
% model setup
%==========================================================================
nlag      = 2;                  % number of lags
nvar      = 11;                   % number of endogenous variables
nex       = 0;                   % set equal to 1 if a constant is included; 0 otherwise
m         = nvar*nlag + nex;     % number of exogenous variables
nd        = 500000;                % number of orthogonal-reduced-form (B,Sigma,Q) draws
iter_show = 5e2;                 % display iteration every iter_show draws
horizon   = 60;                  % maximum horizon for IRFs
index     = 40;                  % define  horizons for the FEVD
horizons  = 0:5;                 % horizons to restrict
NS        = 1 + numel(horizons); % number of objects in F(A_{0},A_{+}) to which we impose sign and zero restrictios: F(THETA)=[A_{0};L_{0},...,L_{horizons}]
e         = eye(nvar);           % create identity matrix
maxdraws  = 1e4;                 % max number of importance sampling draws
conjugate = 'structural';        % structural or irfs or empty
%==========================================================================
% identification: declare Ss and Zs matrices
%==========================================================================
% restrictions on A0  and/or IRFs
% sign restrictions
S = cell(nvar,1);
for ii=1:nvar
    S{ii}=zeros(0,nvar*NS);
end

S{1}(1,nvar+2) = -1;S{1}(5,2*nvar+2)  = -1;S{1}(9,3*nvar+2)  = -1;S{1}(13,4*nvar+2) = -1;S{1}(17,5*nvar+2) = -1;S{1}(21,6*nvar+2) = -1;
S{1}(2,nvar+3) = -1;S{1}(6,2*nvar+3)  = -1;S{1}(10,3*nvar+3) = -1;S{1}(14,4*nvar+3) = -1;S{1}(18,5*nvar+3) = -1;S{1}(22,6*nvar+3) = -1;
S{1}(3,nvar+5) = -1;S{1}(7,2*nvar+5)  = -1;S{1}(11,3*nvar+5) = -1;S{1}(15,4*nvar+5) = -1;S{1}(19,5*nvar+5) = -1;S{1}(23,6*nvar+5) = -1;
S{1}(4,nvar+6) =  1;S{1}(8,2*nvar+6)  =  1;S{1}(12,3*nvar+6) =  1;S{1}(16,4*nvar+6) =  1;S{1}(20,5*nvar+6) =  1;S{1}(24,6*nvar+6) =  1;
S{1}(25,1)        = -1;
S{1}(26,2)        = -1;
S{1}(27,3)        = -1;
S{1}(28,6)        =  1;

% zero restrictions
Z=cell(nvar,1);
for i=1:nvar
    Z{i}=zeros(0,nvar*NS);
end
Z{1}(1,4)=1;
Z{1}(2,5)=1;
Z{1}(3,7)=1; 
Z{1}(4,8)=1; 
Z{1}(5,9)=1; 
Z{1}(6,10)=1;    Z{1}(7,11)=1; 

%==========================================================================
% Setup info
%==========================================================================
info=SetupInfo(nvar,m,Z,@(x)chol(x));

% ZF(A_{0},A_{+})
info.nlag     = nlag;
info.horizons = horizons;
info.ZF       = @(x,y)ZF(x,y);

% functions useful to compute the importance sampler weights
iw_info = info;
fs      = @(x)ff_h(x,iw_info);
r       = @(x)ZeroRestrictions(x,iw_info);

if strcmp(conjugate,'irfs')==1
    fo              = @(x)f_h(x,iw_info);
    fo_str2irfs     = @(x)StructuralToIRF(x,iw_info);
    fo_str2irfs_inv = @(x)IRFToStructural(x,iw_info);
    r_irfs          = @(x)IRFRestrictions_more_general(x,iw_info); 
end

% function useful to check the sign restrictions
fh_S_restrictions  = @(x)SF(x,iw_info,S);

%==========================================================================
% write data in Rubio, Waggoner, and Zha (RES 2010)'s notation
%==========================================================================
% yt(t) A0 = xt(t) Aplus + constant + et(t) for t=1...,T;
% yt(t)    = xt(t) B     + ut(t)            for t=1...,T;
% x(t)     = [yt(t-1), ... , yt(t-nlag), constant];
% matrix notation yt = xt*B + ut;
% xt=[yt_{-1} ones(T,1)];
yt = num(nlag+1:end,:);
T  = size(yt,1);
xt = zeros(T,nvar*nlag+nex);
for i=1:nlag
    xt(:,nvar*(i-1)+1:nvar*i) = num((nlag-(i-1)):end-i,:) ;
end
if nex>=1
    xt(:,nvar*nlag+nex)=ones(T,1);
end
% write data in Zellner (1971, pp 224-227) notation
Y = yt; % T by nvar matrix of observations
X = xt; % T by (nvar*nlag+1) matrix of regressors

% prior for reduced-form parameters
nnuBar              = 0;
OomegaBarInverse    = zeros(m);
PpsiBar             = zeros(m,nvar);
PphiBar             = zeros(nvar);

% posterior for reduced-form parameters
nnuTilde            = T +nnuBar;
OomegaTilde         = (X'*X  + OomegaBarInverse)\eye(m);
OomegaTildeInverse  =  X'*X  + OomegaBarInverse;
PpsiTilde           = OomegaTilde*(X'*Y + OomegaBarInverse*PpsiBar);
PphiTilde           = Y'*Y + PphiBar + PpsiBar'*OomegaBarInverse*PpsiBar - PpsiTilde'*OomegaTildeInverse*PpsiTilde;
PphiTilde           = (PphiTilde'+PphiTilde)*0.5;

% useful definitions
% definitios used to store orthogonal-reduced-form draws, volume elements, and unnormalized weights
Bdraws         = cell([nd,1]); % reduced-form lag parameters
Sigmadraws     = cell([nd,1]); % reduced-form covariance matrices
Qdraws         = cell([nd,1]); % orthogonal matrices
storevefh      = zeros(nd,1);  % volume element f_{h}
storevegfhZ    = zeros(nd,1);  % volume element g o f_{h}|Z
uw             = zeros(nd,1);  % unnormalized importance sampler weights

if strcmp(conjugate,'irfs')==1
    storevephi      = zeros(nd,1);  % volume element f_{h}
    storevegphiZ    = zeros(nd,1);  % volume element g o f_{h}|Z
end

% definitions related to IRFs; based on page 12 of Rubio, Waggoner, and Zha (RES 2010)
J      = [e;repmat(zeros(nvar),nlag-1,1)];
A      = cell(nlag,1);
extraF = repmat(zeros(nvar),1,nlag-1);
F      = zeros(nlag*nvar,nlag*nvar);
for l=1:nlag-1
    F((l-1)*nvar+1:l*nvar,nvar+1:nlag*nvar)=[repmat(zeros(nvar),1,l-1) e repmat(zeros(nvar),1,nlag-(l+1))];
end

% definition to facilitate the draws from B|Sigma
hh              = info.h;
cholOomegaTilde = hh(OomegaTilde)'; % this matrix is used to draw B|Sigma below

% initialize counters to track the state of the computations
counter = 1;
record  = 1;
count   = 0;
tStart = tic;
while record<=nd
    
    % step 1 in Algorithm 2
    Sigmadraw     = iwishrnd(PphiTilde,nnuTilde);
    cholSigmadraw = hh(Sigmadraw)';
    Bdraw         = kron(cholSigmadraw,cholOomegaTilde)*randn(m*nvar,1) + reshape(PpsiTilde,nvar*m,1);
    Bdraw         = reshape(Bdraw,nvar*nlag+nex,nvar);
    % store reduced-form draws
    Bdraws{record,1}     = Bdraw;
    Sigmadraws{record,1} = Sigmadraw;
    
    % steps 2:4 of Algorithm 2
    w           = DrawW(iw_info);   
    x           = [vec(Bdraw); vec(Sigmadraw); w];
    structpara  = ff_h_inv(x,iw_info);
    
    % store the matrix Q associated with step 3
    Qdraw            = SpheresToQ(w,iw_info,Bdraw,Sigmadraw);
    Qdraws{record,1} = reshape(Qdraw,nvar,nvar);
    
    % check if sign restrictions hold
    signs      = fh_S_restrictions(structpara);
    
    if (sum(signs{1}*e(:,1)>0))==size(signs{1}*e(:,1),1)
        
        count=count+1;
        
        % compute importance sampling weights
       
        switch conjugate
            
            case 'structural'
                
                storevefh(record,1)   = (nvar*(nvar+1)/2)*log(2)-(2*nvar+m+1)*LogAbsDet(reshape(structpara(1:nvar*nvar),nvar,nvar));
                storevegfhZ(record,1) = LogVolumeElement(fs,structpara,r); 
                uw(record,1)          = exp(storevefh(record,1) - storevegfhZ(record,1));
                
            case 'irfs'
                
                irfpara                = fo_str2irfs(structpara);
                storevephi(record,1)   = LogVolumeElement(fo,structpara)   + LogVolumeElement(fo_str2irfs_inv,irfpara);
                storevegphiZ(record,1) = LogVolumeElement(fs,structpara,r) + LogVolumeElement(fo_str2irfs_inv,irfpara,r_irfs); 
                uw(record,1)           = exp(storevephi(record,1) - storevegphiZ(record,1));
                
            otherwise
                
                uw(record,1) = 1;
                
        end
        
    else
        
        uw(record,1) = 0;
        
    end
    
    if counter==iter_show
        
        display(['Number of draws = ',num2str(record)])
        display(['Remaining draws = ',num2str(nd-(record))])
        display(['13. UK'])
        counter =0;
        
    end
    counter = counter + 1;
    record=record+1;
end

tElapsed = toc(tStart);
imp_w  = uw/sum(uw);
ne = floor(1/sum(imp_w.^2));

% store draws
Ltilde        = zeros(horizon+1,nvar,nvar,ne); % define array to store IRF
A0tilde       = zeros(nvar,nvar,ne); % define array to store A0
Aplustilde    = zeros(m,nvar,ne); % define array to store Aplus
FEVD          = zeros(nvar,size(index,2),ne);                 % define array to store FEVD
hist_is_draws = zeros(ne,1);   % define array to store draws from importance sampler

for s=1:min(ne,maxdraws)
    
    % draw: B,Sigma,Q
    is_draw = randsample(1:size(imp_w,1),1,true,imp_w);
    hist_is_draws(s,1)=is_draw;

    Bdraw       = Bdraws{is_draw,1};
    Sigmadraw   = Sigmadraws{is_draw,1};
    Qdraw       = Qdraws{is_draw,1};
    
    % compute matrix F: useful for FEVD
    hSigmadraw = hh(Sigmadraw);
    A0         = hSigmadraw\e;
    Aplus      = Bdraw*A0;
    % Page 8 ARRW
    for l=1:nlag-1
        A{l} = Aplus((l-1)*nvar+1:l*nvar,1:end);
        F((l-1)*nvar+1:l*nvar,1:nvar)=A{l}/A0;
    end
    A{nlag} = Aplus((nlag-1)*nvar+1:nlag*nvar,1:end);
    F((nlag-1)*nvar+1:nlag*nvar,:)=[A{nlag}/A0 extraF];
    
    x=[reshape(Bdraw,m*nvar,1); reshape(Sigmadraw,nvar*nvar,1); Qdraw(:)];
    structpara = f_h_inv(x,info);
    
    LIRF =IRF_horizons(structpara, nvar, nlag, m, 0:horizon);
    
    for h=0:horizon
        Ltilde(h+1,:,:,s) =  LIRF(1+h*nvar:(h+1)*nvar,:);
    end
    
    for j=1:size(index,2)
    FEVD(:,j,s)         = variancedecomposition(F',J,Sigmadraw, hh(Sigmadraw)'*Qdraw(:,1),nvar,index(1,j));
    end
    
    A0tilde(:,:,s)    = reshape(structpara(1:nvar*nvar),nvar,nvar);
    Aplustilde(:,:,s) = reshape(structpara(nvar*nvar+1:end),m,nvar);
end

A0tilde    = A0tilde(:,:,1:s);
Aplustilde = Aplustilde(:,:,1:s);
Ltilde     = Ltilde(:,:,:,1:s);
FEVD       = FEVD(:,:,1:s);


cd results
savefile='results_uk.mat';
save(savefile,'Ltilde','A0tilde','Aplustilde','imp_w','ne','FEVD');
cd ..


